/*------------------------------------------------------------------------------
 bin/djv_convert/app.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_CONVERT_APP_H
#define DJV_CONVERT_APP_H

#include <djv_image/app.h>
#include <djv_image/gl_data.h>
#include <djv_image/gl_offscreen.h>
#include <djv_image/image.h>
#include <djv_image/time.h>
#include <djv_base/file.h>

namespace djv_convert {
using namespace djv_base;

//------------------------------------------------------------------------------
// Options

struct Image
{
  Image() :
    scale(1.0),
    channel(djv_image::gl_data::CHANNEL(0))
  {}
  
  Vec2b mirror;
  Vec2f scale;
  djv_image::gl_data::CHANNEL channel;
  Vec2i size;
};

struct Input
{
  Input() :
    layer(0),
    proxy(djv_image::data::PROXY(0)),
    timeout(0)
  {}

  int layer;
  djv_image::data::PROXY proxy;
  Str start, end;
  int timeout;
};
  
struct Output
{
  Output() :
    tag_auto(true)
  {}

  std::auto_ptr<djv_image::PIXEL> pixel;
  djv_image::Tag tag;
  bool tag_auto;
  std::auto_ptr<djv_image::time::FPS> speed;
};

//------------------------------------------------------------------------------
// Application

class App : public djv_image::app::App
{
public:

  App(int, char **);

private:

  bool work();
  
  void cmdln(Str &) throw (Error);
  Str cmdln_help() const;

  File _in, _out;
  Image _image;
  Input _input;
  Output _output;
  std::auto_ptr<djv_image::gl_offscreen::Buffer> _buffer;
  djv_image::gl_data::Data _buffer_data;
};

}

#endif

