/*------------------------------------------------------------------------------
 bin/djv_view/app_cmdln.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include "app.h"

namespace djv_view {

void App::cmdln(Str & in) throw (Error)
{
  DJV_DBG_OBJ("App::cmdln");
  DJV_DBG_PRN("in = " << in);
  
  djv_ui::app::App::cmdln(in);
  if (djv_image::app::App::exit() != djv_base::app::EXIT_DEFAULT) return;

  Str arg;
try
{
  while (! in.empty())
  {
    in >> arg;
    
    // File.
    
    if ("-file_layer" == arg)
    {
      int value = 0;
      in >> value;
      _cmdln.file_layer = std::auto_ptr<int>(new int(value));
    }
    else if ("-file_proxy" == arg)
    {
      djv_image::data::PROXY value(djv_image::data::PROXY(0));
      in >> value;
      _cmdln.file_proxy = std::auto_ptr<djv_image::data::PROXY>(
        new djv_image::data::PROXY(value));
    }
    else if ("-file_cache" == arg)
    {
      bool value = false;
      in >> value;
      _cmdln.file_cache = std::auto_ptr<bool>(new bool(value));
    }
    
    // Window.
    
    else if ("-window_fullscreen" == arg)
      _cmdln.window_fullscreen = std::auto_ptr<bool>(new bool(true));
    else if ("-window_toolbar" == arg)
    {
      bool value = false;
      in >> value;
      _cmdln.window_toolbar = std::auto_ptr<bool>(new bool(value));
    }
    else if ("-window_playbar" == arg)
    {
      bool value = false;
      in >> value;
      _cmdln.window_playbar = std::auto_ptr<bool>(new bool(value));
    }
    
    // View.
    
    else if ("-view_grid" == arg)
    {
      view::GRID value(view::GRID(0));
      in >> value;
      _cmdln.view_grid = std::auto_ptr<view::GRID>(new view::GRID(value));
    }
    else if ("-view_hud" == arg)
    {
      bool value = false;
      in >> value;
      _cmdln.view_hud = std::auto_ptr<bool>(new bool(value));
    }
    
    // Playback.
    
    else if ("-playback" == arg)
    {
      playback::PLAYBACK value(playback::PLAYBACK(0));
      in >> value;
      _cmdln.playback = std::auto_ptr<playback::PLAYBACK>(
        new playback::PLAYBACK(value));
    }
    else if ("-playback_frame" == arg)
    {
      int value = 0;
      in >> value;
      _cmdln.playback_frame = std::auto_ptr<int>(new int(value));
    }
    else if ("-playback_speed" == arg)
    {
      djv_image::time::FPS value(djv_image::time::FPS(0));
      in >> value;
      _cmdln.playback_speed = std::auto_ptr<djv_image::time::Speed>(
        new djv_image::time::Speed(value));
    }
    else if ("-playback_timer" == arg)
    {
      playback::TIMER value(playback::TIMER(0));
      in >> value;
      playback::global()->timer(value);
    }
    else if ("-playback_timer_resolution" == arg)
    {
      double value = 0.0;
      in >> value;
      playback::global()->timer_resolution(value);
    }
    
    // Arguments.
    
    else _input += arg;
  }
}
catch (Str) { throw Error(arg); }
}

Str App::cmdln_help() const
{
  return str::Format(label_cmdln_help).
    arg(djv_image::data::label_proxy(), ", ").
    arg(str::label_bool(), ", ").
    arg(str::label_bool(), ", ").
    arg(str::label_bool(), ", ").
    arg(view::label_grid(), ", ").
    arg(str::label_bool(), ", ").
    arg(playback::label_playback(), ", ").
    arg(djv_image::time::label_fps(), ", ").
    arg(playback::label_timer(), ", ").
    arg(str::label(playback::global()->timer())).
    arg(str::label(playback::global()->timer_resolution())).
    arg(djv_ui::app::App::cmdln_help());
}

const Str label_cmdln_help =
"\n"
"djv_view\n"
"--------\n"
"\n"
"Image and movie viewing application.\n"
"\n"
"Usage\n"
"-----\n"
"\n"
"djv_view [image]... [option]...\n"
"\n"
"Options\n"
"-------\n"
"\n"
"File:\n"
"\n"
"* -file_layer (value)\n"
"  Image layer.\n"
"\n"
"* -file_proxy (value)\n"
"  Proxy scale: %%.\n"
"\n"
"* -file_cache (value)\n"
"  Cache: %%.\n"
"\n"
"Window:\n"
"\n"
"* -window_fullscreen\n"
"  Fullscreen.\n"
"\n"
"* -window_toolbar (value)\n"
"  Toolbar controls: %%.\n"
"\n"
"* -window_playbar (value)\n"
"  Playbar controls: %%.\n"
"\n"
"View:\n"
"\n"
"* -view_grid (value)\n"
"  Grid overlay: %%.\n"
"\n"
"* -view_hud (value)\n"
"  Heads up display: %%.\n"
"\n"
"Playback:\n"
"\n"
"* -playback (value)\n"
"  Playback: %%.\n"
"\n"
"* -playback_frame (value)\n"
"  Frame.\n"
"\n"
"* -playback_speed (value)\n"
"  Speed: %%.\n"
"\n"
"* -playback_timer (value)\n"
"  Timer: %%. Value: %%.\n"
"\n"
"* -playback_timer_resolution (value)\n"
"  Timer resolution (seconds): %%.\n"
"\n"
"General Options\n"
"---------------\n"
"\n"
"%%"
"\n";

}

