/*------------------------------------------------------------------------------
 bin/djv_view/file.h
 
 Files
 -----

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_FILE_H
#define DJV_VIEW_FILE_H

#include "image_cache.h"
#include <djv_ui/layout_row.h>
#include <djv_ui/menu.h>
#include <djv_image/io.h>
#include <djv_base/file.h>

namespace djv_view {
namespace file {
using namespace djv_base;

//------------------------------------------------------------------------------
// File Group

class Group : public djv_ui::Callback_Base
{
public:

  Group(
    djv_ui::menu::Widget *,
    djv_ui::layout::Horizontal * toolbar,
    const Group * copy = 0
  );
  ~Group();
  void del();

  // Value.
  
  void open(const File &);

  const djv_image::Image * get(int frame);
  const File & file() const;
  const List<djv_image::Info> & info() const;
  const djv_image::Tag & tag() const;
  const djv_image::Time & time() const;
  
  void layer(int);
  void proxy(djv_image::data::PROXY);
  void cache(bool);
  int layer() const;
  djv_image::data::PROXY proxy() const;
  bool cache() const;
  List<int> cache_list() const;
  
  // Signals.
  
  djv_ui::Signal<const File &> open_sig;
  djv_ui::Signal<bool> reload_sig;
  djv_ui::Signal<const File &> save_sig, save_frame_sig;
  djv_ui::Signal<bool> image_sig, update_sig;

  // Widget.
  
  void menu_update(djv_ui::menu::Group *);
  void toolbar_update();

private:

  DJV_CALLBACK(Group, open_callback, const File &);
  DJV_CALLBACK(Group, open_callback, bool);
  DJV_CALLBACK(Group, recent_callback, bool);
  DJV_CALLBACK(Group, recent_callback, const List<Str> &);
  DJV_CALLBACK(Group, reload_callback, bool);
  DJV_CALLBACK(Group, reload_callback, djv_image::io::Plugin *);
  DJV_CALLBACK(Group, close_callback, bool);
  DJV_CALLBACK(Group, save_callback, const File &);
  DJV_CALLBACK(Group, save_check_callback, bool);
  DJV_CALLBACK(Group, save_frame_callback, const File &);
  DJV_CALLBACK(Group, save_frame_check_callback, bool);
  DJV_CALLBACK(Group, layer_callback, int);
  DJV_CALLBACK(Group, proxy_callback, djv_image::data::PROXY);
  DJV_CALLBACK(Group, cache_callback, bool);
  DJV_FL_WIDGET_CALLBACK(Group, _open_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _recent_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _reload_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _close_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _save_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _save_frame_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _layer_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _layer_prev_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _layer_next_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _proxy_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _cache_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _messages_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _prefs_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _info_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _about_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _exit_callback);
  
  void cache_update();

  File _file;
  File _file_save;
  List<djv_image::Info> _info;
  djv_image::Tag _tag;
  djv_image::Time _time;
  std::auto_ptr<djv_image::io::Load> _load;
  const djv_image::Image * _image;
  int _layer;
  djv_image::data::PROXY _proxy;
  List<Str> _layer_list;
  bool _cache;
  image_cache::Ref * _cache_ref;
  
  struct _Group;
  std::auto_ptr<_Group> _p;
};

//------------------------------------------------------------------------------
// File Save

struct Save_Info
{
  Save_Info(
    const File & = File(),
    const File & = File(),
    const djv_image::Info & = djv_image::Info(),
    int layer = 0,
    djv_image::data::PROXY = djv_image::data::PROXY(0),
    const djv_image::Time & = djv_image::Time(),
    const djv_image::gl_data::Option & = djv_image::gl_data::Option()
  );
  
  File in, out;
  djv_image::Info info;
  int layer;
  djv_image::data::PROXY proxy;
  djv_image::Time time;
  djv_image::gl_data::Option option;
};

class Save : public djv_ui::Callback_Base
{
public:

  ~Save();
  
  void set(const Save_Info &);

private:

  class _Save;
  
  void del(_Save *);
  
  List<_Save *> _list;
};

//------------------------------------------------------------------------------
// Preferences

class Prefs : public djv_ui::Callback_Base
{
public:

  Prefs();
  ~Prefs();
  
  // Options.
  
  void seq_auto(bool);
  void cmdln_combine(bool);
  bool seq_auto() const;
  bool cmdln_combine() const;

  // Recent files.
  
  void recent(const Str &);
  const List<Str> & recent() const;
  djv_ui::Signal<const List<Str> &> recent_sig;

  // Proxy.
  
  void proxy(djv_image::data::PROXY);
  djv_image::data::PROXY proxy() const;
  djv_ui::Signal<djv_image::data::PROXY> proxy_sig;

  // Cache.
  
  void cache(bool);
  void cache_size(int);
  void cache_display(bool);
  bool cache() const;
  int cache_size() const;
  bool cache_display() const;
  djv_ui::Signal<bool> cache_sig;

private:

  bool _seq_auto;
  bool _cmdln_combine;
  List<Str> _recent;
  djv_image::data::PROXY _proxy;
  bool _cache;
  int _cache_size;
  bool _cache_display;
};

//------------------------------------------------------------------------------
// Preferences Widget

class Prefs_Widget : public djv_ui::widget::Widget
{
public:

  Prefs_Widget();
  
private:

  DJV_CALLBACK(Prefs_Widget, seq_auto_callback, bool);
  DJV_CALLBACK(Prefs_Widget, cmdln_combine_callback, bool);
  DJV_CALLBACK(Prefs_Widget, proxy_callback, int);
  DJV_CALLBACK(Prefs_Widget, cache_callback, bool);
  DJV_CALLBACK(Prefs_Widget, cache_size_callback, int);
  DJV_CALLBACK(Prefs_Widget, cache_display_callback, bool);
};

//------------------------------------------------------------------------------
// Labels

extern const Str
  menu_title,
  menu_open,
  menu_recent,
  menu_reload,
  menu_close,
  menu_save,
  menu_save_frame,
  menu_layer,
  menu_layer_prev,
  menu_layer_next,
  menu_proxy,
  menu_cache;

extern const Str
  tooltip_open,
  tooltip_reload,
  tooltip_close;

extern const Str
  prefs_title,
  prefs_group,
  prefs_seq_auto,
  prefs_cmdln_combine,
  prefs_proxy_group,
  prefs_proxy,
  prefs_proxy_text,
  prefs_cache_group,
  prefs_cache,
  prefs_cache_size,
  prefs_cache_size_unit,
  prefs_cache_display,
  prefs_cache_text;

extern const Str
  dialog_save_check;

//------------------------------------------------------------------------------
// Global

Save * save();
Prefs * prefs();

}
}

#endif

