/*------------------------------------------------------------------------------
 bin/djv_view/file_prefs_widget.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include "file.h"
#include "image_cache.h"
#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/menu.h>
#include <djv_ui/num_int.h>
#include <djv_ui/style.h>
#include <djv_ui/text.h>

namespace djv_view {
namespace file {

namespace {

class Cache_Size_Widget : public djv_ui::widget::Widget
{
public:

  Cache_Size_Widget() :
    sig(this)
  {
    using namespace djv_ui;
  
    _widget = new num::Int_Edit;
    _widget->range(0, _widget->max());
    _widget->inc(100, 1000);
    menu::Popup * menu = new menu::Popup(image_cache::label_size());
    
    size(layout::FIXED, layout::FIXED);
    Layout_H * layout = new Layout_H(this);
    layout->margin(0);
    layout->add(_widget);
    layout->add(menu);
    
    _widget->set(prefs()->cache_size());
    
    _widget->sig.set(this, widget_callback);
    menu->sig.set(this, menu_callback);
  }
  
  void set(int in)
  {
    _widget->set(in);
  }
  
  djv_ui::Signal<int> sig;
  
private:

  DJV_CALLBACK(Cache_Size_Widget, widget_callback, int) { sig.emit(in); }
  DJV_CALLBACK(Cache_Size_Widget, menu_callback, int)
    { _widget->set(image_cache::default_size()[in]); }
  
  djv_ui::num::Int_Edit * _widget;
};

}

Prefs_Widget::Prefs_Widget()
{
  using namespace djv_ui;
  
  // Widgets.

  group::Box * group =
    new group::Box(prefs_group);
  button::Check * seq_auto =
    new button::Check(prefs_seq_auto);
  button::Check * cmdln_combine =
    new button::Check(prefs_cmdln_combine);

  group::Box * proxy_group =
    new group::Box(prefs_proxy_group);
  choice::Widget * proxy =
    new choice::Widget(djv_image::data::label_proxy(), prefs_proxy);
  label::Text * proxy_text =
    new label::Text(prefs_proxy_text);

  group::Box * cache_group =
    new group::Box(prefs_cache_group);
  button::Check * cache =
    new button::Check(prefs_cache);
  Cache_Size_Widget * cache_size =
    new Cache_Size_Widget;
  label::Widget * cache_size_label =
    new label::Widget(prefs_cache_size);
  label::Widget * cache_size_unit_label =
    new label::Widget(prefs_cache_size_unit);
  button::Check * cache_display =
    new button::Check(prefs_cache_display);
  label::Text * cache_text =
    new label::Text(prefs_cache_text);

  // Layout.
  
  Layout_V * layout, * layout_group;
  Layout_H * layout_h;
  
  layout = new Layout_V(this);

  layout->add(group);
  layout_group = new Layout_V(group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(seq_auto);
  layout_group->add(cmdln_combine);

  layout->add(proxy_group);
  layout_group = new Layout_V(proxy_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(proxy);
  layout_group->add(proxy_text);

  layout->add(cache_group);
  layout_group = new Layout_V(cache_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(cache);
  layout_h = new Layout_H(layout_group);
  layout_h->margin(0);
  layout_h->add(cache_size_label);
  layout_h->add(cache_size);
  layout_h->add(cache_size_unit_label);
  layout_group->add(cache_display);
  layout_group->add(cache_text);

  layout->add_stretch();

  // Initialize.
  
  seq_auto->set(prefs()->seq_auto());
  cmdln_combine->set(prefs()->cmdln_combine());
  proxy->set(prefs()->proxy());
  cache->set(prefs()->cache());
  cache_size->set(prefs()->cache_size());
  cache_display->set(prefs()->cache_display());
  
  // Callbacks.

  seq_auto->sig.set(this, seq_auto_callback);
  cmdln_combine->sig.set(this, cmdln_combine_callback);
  proxy->sig.set(this, proxy_callback);
  cache->sig.set(this, cache_callback);
  cache_size->sig.set(this, cache_size_callback);
  cache_display->sig.set(this, cache_display_callback);
}

void Prefs_Widget::seq_auto_callback(bool in)
  { prefs()->seq_auto(in); }
void Prefs_Widget::cmdln_combine_callback(bool in)
  { prefs()->cmdln_combine(in); }

void Prefs_Widget::proxy_callback(int in)
  { prefs()->proxy(djv_image::data::PROXY(in)); }

void Prefs_Widget::cache_callback(bool in)
  { prefs()->cache(in); }

void Prefs_Widget::cache_size_callback(int in)
  { prefs()->cache_size(in); }

void Prefs_Widget::cache_display_callback(bool in)
  { prefs()->cache_display(in); }

}
}

