/*------------------------------------------------------------------------------
 bin/djv_view/image.h
 
 Images
 ------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_IMAGE_H
#define DJV_VIEW_IMAGE_H

#include <djv_ui/image_io.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/menu.h>
#include <djv_image/gl_data.h>
#include <djv_image/image.h>
#include <djv_base/file.h>

namespace djv_view {
namespace image {
using namespace djv_base;

//------------------------------------------------------------------------------
// Options

enum SCALE
{
  SCALE_NONE,
  SCALE_16_9,
  SCALE_1_85,
  SCALE_2_35
};

enum ROTATE
{
  ROTATE_0,
  ROTATE_90,
  ROTATE_180,
  ROTATE_270
};

using djv_image::gl_data::CHANNEL;

struct Display : public djv_image::gl_data::Display
{
  Str name;
  File lut_file;
};

//------------------------------------------------------------------------------
// Image Group

class Group : public djv_ui::Callback_Base
{
public:
  
  Group(
    djv_ui::menu::Widget *,
    djv_ui::layout::Horizontal * toolbar,
    const Group * copy = 0
  );
  ~Group();

  // Value.
  
  void pick(const Vec2i &, double zoom);
  
  void display(const Display &);
  bool frame_store() const;
  const Vec2b & mirror() const;
  SCALE scale() const;
  ROTATE rotate() const;
  bool profile() const;
  CHANNEL channel() const;
  const Display & display() const;
  
  // Signals.
  
  djv_ui::Signal<bool> redraw_sig, resize_sig;
  djv_ui::Signal<bool> frame_store_sig;
  djv_ui::Signal<bool> display_sig;
  djv_ui::Signal<const Display &> display_value_sig;
  djv_ui::Signal<bool> magnify_sig, sample_sig, histogram_sig, info_sig,
    update_sig;
  
  // Widget.
  
  void menu_update(djv_ui::menu::Group *);
  void toolbar_update();

private:

  DJV_CALLBACK(Group, frame_store_callback, bool);
  DJV_CALLBACK(Group, frame_store_load_callback, bool);
  DJV_CALLBACK(Group, mirror_callback, const Vec2b &);
  DJV_CALLBACK(Group, scale_callback, SCALE);
  DJV_CALLBACK(Group, rotate_callback, ROTATE);
  DJV_CALLBACK(Group, profile_callback, bool);
  DJV_CALLBACK(Group, channel_callback, CHANNEL);
  DJV_CALLBACK(Group, display_callback, bool);
  DJV_CALLBACK(Group, display_callback, const Display &);
  DJV_CALLBACK(Group, display_reset_callback, bool);
  DJV_CALLBACK(Group, display_add_callback, const Str &);
  DJV_CALLBACK(Group, display_del_callback, int);
  DJV_CALLBACK(Group, display_del_all_callback, bool);
  DJV_CALLBACK(Group, display_list_callback, bool);
  DJV_CALLBACK(Group, magnify_callback, bool);
  DJV_CALLBACK(Group, sample_callback, bool);
  DJV_CALLBACK(Group, histogram_callback, bool);
  DJV_CALLBACK(Group, info_callback, bool);
  DJV_FL_WIDGET_CALLBACK(Group, _frame_store_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _frame_store_load_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _mirror_h_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _mirror_v_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _scale_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _rotate_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _profile_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _channel_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _display_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _display_reset_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _display_add_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _display_del_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _display_del_all_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _display_list_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _magnify_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _sample_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _histogram_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _info_callback);

  bool _frame_store;
  Vec2b _mirror;
  SCALE _scale;
  ROTATE _rotate;
  bool _profile;
  CHANNEL _channel;
  Display _display;
  
  struct _Group;
  std::auto_ptr<_Group> _p;
};

//------------------------------------------------------------------------------
// Preferences

class Prefs : public djv_ui::Callback_Base
{
public:

  Prefs();
  ~Prefs();
  
  // Frame store.
  
  void frame_store_file_reload(bool);
  bool frame_store_file_reload() const;
  
  // Transforms.
  
  void mirror(const Vec2b &);
  void scale(image::SCALE);
  void rotate(image::ROTATE);
  const Vec2b & mirror() const;
  image::SCALE scale() const;
  image::ROTATE rotate() const;
  djv_ui::Signal<const Vec2b &> mirror_sig;
  djv_ui::Signal<image::SCALE> scale_sig;
  djv_ui::Signal<image::ROTATE> rotate_sig;
  
  // Color profile.
  
  void profile(bool);
  bool profile() const;
  djv_ui::Signal<bool> profile_sig;

  // Image channel.
  
  void channel(image::CHANNEL);
  image::CHANNEL channel() const;
  djv_ui::Signal<image::CHANNEL> channel_sig;

  // Image display.
  
  void display(int);
  int display() const;

  image::Display display_value() const;

  void display_list(const List<image::Display> &);
  const List<image::Display> & display_list() const;
  List<Str> display_name_list() const;
  
  djv_ui::Signal<int> display_sig;
  djv_ui::Signal<const image::Display &> display_value_sig;
  djv_ui::Signal<bool> display_list_sig;

private:

  bool _frame_store_file_reload;
  Vec2b _mirror;
  image::SCALE _scale;
  image::ROTATE _rotate;
  bool _profile;
  image::CHANNEL _channel;
  int _display;
  List<image::Display> _display_list;
};

//------------------------------------------------------------------------------
// Preferences Widget

class Prefs_Widget : public djv_ui::widget::Widget
{
public:

  Prefs_Widget();
  ~Prefs_Widget();
  
private:

  DJV_CALLBACK(Prefs_Widget, frame_store_file_reload_callback, bool);
  DJV_CALLBACK(Prefs_Widget, mirror_h_callback, bool);
  DJV_CALLBACK(Prefs_Widget, mirror_v_callback, bool);
  DJV_CALLBACK(Prefs_Widget, scale_callback, int);
  DJV_CALLBACK(Prefs_Widget, rotate_callback, int);
  DJV_CALLBACK(Prefs_Widget, profile_callback, bool);
  DJV_CALLBACK(Prefs_Widget, channel_callback, int);
  DJV_CALLBACK(Prefs_Widget, display_callback, int);
  DJV_CALLBACK(Prefs_Widget, display_list_callback, bool);
  
  void widget_update();
  
  struct _Prefs_Widget;
  std::auto_ptr<_Prefs_Widget> _p;
};

//------------------------------------------------------------------------------
// Functions

Vec2f scale(SCALE, const Vec2i &);

double rotate(ROTATE);

const djv_image::Image * lut_load(const File &);

//------------------------------------------------------------------------------
// Conversion

Str & operator >> (Str &, SCALE &) throw (Str);
Str & operator >> (Str &, ROTATE &) throw (Str);
Str & operator >> (Str &, Display &) throw (Str);
Str & operator << (Str &, SCALE);
Str & operator << (Str &, ROTATE);
Str & operator << (Str &, const Display &);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_scale();
const List<Str> & label_rotate();
using djv_image::gl_data::label_channel;
  
extern const Str
  menu_title,
  menu_frame_store,
  menu_frame_store_load,
  menu_mirror_h,
  menu_mirror_v,
  menu_scale,
  menu_rotate,
  menu_profile,
  menu_channel,
  menu_display,
  menu_display_show,
  menu_display_reset,
  menu_display_add,
  menu_display_del,
  menu_display_del_all,
  menu_magnify,
  menu_sample,
  menu_histogram,
  menu_info;

extern const Str
  toolbar_pick,
  toolbar_pick_format;

extern const Str
  tooltip_pick,
  tooltip_display,
  tooltip_magnify,
  tooltip_sample;
  
extern const Str
  prefs_title,
  prefs_frame_store_group,
  prefs_frame_store_file_reload,
  prefs_xform_group,
  prefs_xform_mirror_h,
  prefs_xform_mirror_v,
  prefs_xform_scale,
  prefs_xform_rotate,
  prefs_color_group,
  prefs_color_profile,
  prefs_color_channel,
  prefs_color_display,
  prefs_color_display_default;

extern const Str
  dialog_display_add,
  dialog_display_del,
  dialog_display_del_all;

//------------------------------------------------------------------------------
// Global

Prefs * prefs();

}
}

#endif

