/*------------------------------------------------------------------------------
 bin/djv_view/image_display.h
 
 Image Display
 -------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_IMAGE_DISPLAY_H
#define DJV_VIEW_IMAGE_DISPLAY_H

#include "image.h"
#include <djv_ui/dialog.h>

namespace djv_view {
namespace image_display {
using namespace djv_base;

//------------------------------------------------------------------------------
// Dialog

class Dialog : public djv_ui::dialog::Base
{
public:
  
  Dialog();
  ~Dialog();
  void del() { Base::del(); }

  // Value.
  
  void set(const void *, const image::Display &);
  void pick(const void *, const image::Display &);
  void update(const void *, const image::Display &);
  void del(const void *);
  djv_ui::Signal<const image::Display &> sig;

  // Widget.
  
  void show();
  
private:

  DJV_CALLBACK(Dialog, widget_callback, const image::Display &);
  DJV_CALLBACK(Dialog, lut_callback, const File &);
  DJV_CALLBACK(Dialog, brightness_callback, double);
  DJV_CALLBACK(Dialog, contrast_callback, double);
  DJV_CALLBACK(Dialog, saturation_callback, double);
  DJV_CALLBACK(Dialog, levels_in0_callback, double);
  DJV_CALLBACK(Dialog, levels_in1_callback, double);
  DJV_CALLBACK(Dialog, gamma_callback, double);
  DJV_CALLBACK(Dialog, levels_out0_callback, double);
  DJV_CALLBACK(Dialog, levels_out1_callback, double);
  DJV_CALLBACK(Dialog, soft_clip_callback, double);
  DJV_CALLBACK(Dialog, close_callback, bool);

  void widget_update();
  
  const void * _obj;
  image::Display _value;
  
  struct _Dialog;
  std::auto_ptr<_Dialog> _p;
};

//------------------------------------------------------------------------------
// Labels

extern const Str
  dialog_title,
  dialog_lut,
  dialog_color,
  dialog_brightness,
  dialog_contrast,
  dialog_saturation,
  dialog_levels,
  dialog_levels_in,
  dialog_gamma,
  dialog_levels_out,
  dialog_soft_clip;

//------------------------------------------------------------------------------
// Global

Dialog * dialog();

}
}

#endif

