/*------------------------------------------------------------------------------
 bin/djv_view/image_etc.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include "app.h"
#include "image.h"
#include <djv_image/io.h>

namespace djv_view {
namespace image {

const djv_image::Image * lut_load(const File & in)
{
  if (in.get().empty()) return 0;
  
  DJV_DBG_OBJ("lut_load");
  DJV_DBG_PRN("in = " << in);
  
  File file(in);
  if (file.seq_valid()) file.type = djv_base::file::SEQ;
  
  const djv_image::Image * out = 0;
  try
  {
    out = djv_image::io::global()->load(file)->get();
    DJV_DBG_PRN("image = " << *out);
  }
  catch (Error error)
  {
    app()->error(error, file.name());
  }
  return out;
}

Vec2f scale(SCALE in, const Vec2i & size)
{
  static const double data [] = { 1.0, 16.0 / 9.0, 1.85, 2.35 };
  Vec2f out(1.0);
  switch (in)
  {
    case SCALE_16_9:
    case SCALE_1_85:
    case SCALE_2_35: out.x = data[in] / vec::aspect(size); break;
    default: break;
  }
  return out;
}

double rotate(ROTATE in)
{
  static const double data [] = { 0, 90, 180, 270 };
  return data[in];
}

Str & operator >> (Str & in, Display & out) throw (Str)
{
  in >> out.name;

  Str tmp;
  in >> tmp;
  out.lut_file = tmp;

  in >> out.color;
  in >> out.levels;
  in >> out.soft_clip;
  return in;
}

Str & operator << (Str & out, const Display & in)
{
  return out <<
    in.name <<
    in.lut_file <<
    in.color <<
    in.levels <<
    in.soft_clip;
}

DJV_STR_OPERATOR_SET(SCALE, label_scale())
DJV_STR_OPERATOR_GET(SCALE, label_scale())
DJV_STR_OPERATOR_SET(ROTATE, label_rotate())
DJV_STR_OPERATOR_GET(ROTATE, label_rotate())

const List<Str> & label_scale()
{
  static const List<Str> data = List<Str>() <<
    "None" <<
    "16.9" <<
    "1.85" <<
    "2.35";
  return data;
}

const List<Str> & label_rotate()
{
  static const List<Str> data = List<Str>() <<
    "0" <<
    "90" <<
    "180" <<
    "270";
  return data;
}

const Str
  menu_title = "Im&age",
  menu_frame_store = "Frame Store",
  menu_frame_store_load = "Frame Store Load",
  menu_mirror_h = "Mirror Horizontal",
  menu_mirror_v = "Mirror Vertical",
  menu_scale = "Scale",
  menu_rotate = "Rotate",
  menu_profile = "Profile",
  menu_channel = "Channel",
  menu_display = "Display",
  menu_display_show = "Show",
  menu_display_reset = "Reset",
  menu_display_add = "Add",
  menu_display_del = "Delete",
  menu_display_del_all = "Delete All",
  menu_magnify = "Magnify",
  menu_sample = "Sample",
  menu_histogram = "Histogram",
  menu_info = "Information";

const Str
  toolbar_pick = "Pixel: %% %%",
  toolbar_pick_format = "Pixel: 0000 0000";

const Str
  tooltip_pick = "Image pixel.",
  tooltip_display = "Show image display dialog.\nKeyboard shortcut: %%",
  tooltip_magnify = "Show image magnify dialog.\nKeyboard shortcut: %%",
  tooltip_sample = "Show image sample dialog.\nKeyboard shortcut: %%";

const Str
  prefs_title = "Image",
  prefs_frame_store_group = "Frame Store",
  prefs_frame_store_file_reload = "Store frame on file reload.",
  prefs_xform_group = "Transform",
  prefs_xform_mirror_h = "Mirror horizontal.",
  prefs_xform_mirror_v = "Mirror vertical.",
  prefs_xform_scale = "Scale:",
  prefs_xform_rotate = "Rotate:",
  prefs_color_group = "Color",
  prefs_color_profile = "Profile.",
  prefs_color_channel = "Channel:",
  prefs_color_display = "Display:",
  prefs_color_display_default = "Default";

const Str
  dialog_display_add = "Add display setting:",
  dialog_display_del = "Delete display setting:",
  dialog_display_del_all = "Delete all display settings?";

}
}

