/*------------------------------------------------------------------------------
 bin/djv_view/image_histogram.h
 
 Image Histogram
 ---------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_IMAGE_HISTOGRAM_H
#define DJV_VIEW_IMAGE_HISTOGRAM_H

#include "view.h"
#include <djv_ui/dialog.h>
#include <djv_ui/group.h>

namespace djv_view {
namespace image_histogram {
using namespace djv_base;

//------------------------------------------------------------------------------
// Widget

class Widget : public djv_ui::group::Frame
{
public:

  Widget();
  
  // Value.
  
  void set(
    const djv_image::Data *,
    const djv_image::Color & min,
    const djv_image::Color & max
  );
  
  // Widget.
  
  void draw();
  
  // Layout.
  
  void dirty();
  
private:

  const djv_image::Data * _value;
  int _value_max;
  djv_image::Color _min, _max;
};

//------------------------------------------------------------------------------
// Dialog

class Dialog : public djv_ui::dialog::Base
{
public:
  
  Dialog();
  ~Dialog();
  void del();

  // Value.
  
  void set(const view::Widget *);
  void pick(const view::Widget *);
  void update(const view::Widget *);
  void del(const view::Widget *);
  
  // Widget.
  
  void show();
  
  // Layout.
  
  void update() { Base::update(); }
  
private:

  DJV_CALLBACK(Dialog, size_callback, int);
  DJV_CALLBACK(Dialog, close_callback, bool);
  
  void widget_update();
  
  const view::Widget * _view;
  djv_image::gl_data::HISTOGRAM _size;
  djv_image::Data _histogram;
  djv_image::Color _min, _max;
  
  struct _Dialog;
  std::auto_ptr<_Dialog> _p;
};

//------------------------------------------------------------------------------
// Labels

extern const Str
  dialog_title,
  dialog_size,
  dialog_min,
  dialog_max;

//------------------------------------------------------------------------------
// Global

Dialog * dialog();

}
}

#endif

