/*------------------------------------------------------------------------------
 bin/djv_view/playback_group.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include "playback.h"
#include "shortcut.h"
#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/num.h>
#include <djv_ui/num_float.h>
#include <djv_ui/shuttle.h>
#include <djv_ui/slider.h>
#include <djv_ui/text.h>

namespace djv_view {
namespace playback {

struct Group::_Group
{
  _Group() :
    menu(0)
  {}

  djv_ui::menu::Widget * menu;
  
  djv_ui::button::Tool * stop, * forward, * reverse;
  djv_ui::shuttle::Widget * playback_shuttle;
 
  widget::Frame_Widget * frame;
  widget::Frame_Slider * frame_slider;
  djv_ui::shuttle::Widget * frame_shuttle;
  
  widget::Frame_Display_Widget * start, * end;
  djv_ui::button::Tool * start_button, * end_button;
  djv_ui::button::Tool * prev, * next;
  
  widget::Frame_Widget * in, * out;
  djv_ui::button::Tool * in_mark, * in_reset;
  djv_ui::button::Tool * out_mark, * out_reset;
  
  widget::Frame_Display_Widget * duration;
  
  widget::Speed_Widget * speed;
  widget::Speed_Display_Widget * speed_real;
  djv_ui::button::Tool * every_frame;
};

Group::Group(
  djv_ui::menu::Widget * menu,
  djv_ui::group::Frame * playbar,
  Group * copy
) :
  image_sig(this),
  overlay_sig(this),
  update_sig(this),
  _playback(copy ? copy->_playback : STOP),
  _time(copy ? copy->_time : djv_image::Time()),
  _frame(copy ? copy->_frame : 0),
  _frame_tmp(0),
  _in(copy ? copy->_in : 0),
  _out(copy ? copy->_out : 0),
  _every_frame(copy ? copy->_every_frame : prefs()->every_frame()),
  _shuttle(false),
  _shuttle_speed(0.0),
  _speed(copy ? copy->_speed : djv_image::time::Speed()),
  _speed_real(0.0),
  _dropped_frames(false),
  _dropped_frames_tmp(false),
  _idle_init(true),
  _idle_frame(0),
  _speed_counter(0),
  _p(new _Group)
{
  using namespace djv_ui;

  // Widgets.
  
  _p->menu = menu;
  
  // Playback widgets.

  _p->stop = new button::Tool("playback_stop");
  _p->stop->type(button::Tool::RADIO);
  _p->forward = new button::Tool("playback_forward");
  _p->forward->type(button::Tool::RADIO);
  _p->reverse = new button::Tool("playback_reverse");
  _p->reverse->type(button::Tool::RADIO);

  button::radio_group(List<button::Tool *>() << _p->stop << _p->forward <<
    _p->reverse);

  _p->playback_shuttle = new shuttle::Widget("playback_");

  // Frame widgets.
  
  _p->frame = new widget::Frame_Widget;
  _p->frame_slider = new widget::Frame_Slider;
  _p->frame_shuttle = new shuttle::Widget("playback_frame_");

  _p->start = new widget::Frame_Display_Widget;
  _p->start_button = new button::Tool("playback_start");
  _p->end = new widget::Frame_Display_Widget;
  _p->end_button = new button::Tool("playback_end");
  
  _p->prev = new button::Tool("playback_prev");
  _p->prev->type(button::Tool::REPEAT);
  _p->next = new button::Tool("playback_next");
  _p->next->type(button::Tool::REPEAT);
  
  _p->duration = new widget::Frame_Display_Widget;

  // In/out point widgets.
  
  _p->in = new widget::Frame_Widget;
  _p->in->text_color(FL_SELECTION_COLOR);
  _p->in_mark = new button::Tool("tool_start");
  _p->in_reset = new button::Tool("tool_x");
  
  _p->out = new widget::Frame_Widget;
  _p->out->text_color(FL_SELECTION_COLOR);
  _p->out_mark = new button::Tool("tool_end");
  _p->out_reset = new button::Tool("tool_x");
  
  // Speed widgets.
  
  _p->speed = new widget::Speed_Widget;
  _p->speed_real = new widget::Speed_Display_Widget;
  
  _p->every_frame = new button::Tool("tool_lock");
  _p->every_frame->type(button::Tool::TOGGLE);
  
  // Layout.
  
  Layout_H * layout = new Layout_H(playbar);
  layout->spacing(0);
  layout->margin(0);
  
  Layout_V * left_layout = new Layout_V(layout);
  left_layout->margin(0);
  Layout_H * layout_h = new Layout_H(left_layout);
  layout_h->spacing(0);
  layout_h->margin(0);
  layout_h->add(_p->reverse);
  layout_h->add(_p->stop);
  layout_h->add(_p->forward);
  layout_h->add(_p->playback_shuttle);
  layout_h->add(_p->start_button);
  layout_h->add(_p->prev);
  layout_h->add(_p->next);
  layout_h->add(_p->end_button);
  layout_h->add(_p->frame_shuttle);
  left_layout->add_stretch();
  layout_h = new Layout_H(left_layout);
  layout_h->add(_p->speed);
  layout_h->add(_p->speed_real);
  layout_h->add(_p->every_frame);
  layout_h->add_stretch();
  
  Layout_V * right_layout = new Layout_V(layout);
  layout->stretch(right_layout);
  layout_h = new Layout_H(right_layout);
  layout_h->margin(0);
  layout_h->add(_p->frame_slider);
  layout_h->stretch(_p->frame_slider);
  right_layout->add_spacer(-1, true);
  layout_h = new Layout_H(right_layout);
  layout_h->margin(0);
  layout_h->add(_p->frame);
  layout_h->add(_p->in_mark);
  layout_h->add(_p->in_reset);
  layout_h->add(_p->in);
  layout_h->add(_p->start);
  layout_h->add_spacer(-1, true);
  layout_h->add(_p->end);
  layout_h->add(_p->out);
  layout_h->add(_p->out_reset);
  layout_h->add(_p->out_mark);
  layout_h->add(_p->duration);
  
  // Initialize.
  
  playback_update();
  time_update();
  speed_update();
  frame_update();
  
  // Playback callbacks.
  
  _p->stop->sig.set(this, stop_callback);
  _p->forward->sig.set(this, forward_callback);
  _p->reverse->sig.set(this, reverse_callback);
  
  _p->playback_shuttle->sig.set(this, playback_shuttle_callback);
  _p->playback_shuttle->value_sig.set(this, playback_shuttle_value_callback);
  
  // Frame callbacks.
  
  _p->frame->sig.set(this, frame_callback);
  _p->frame_slider->sig.set(this, frame_slider_callback);
  _p->frame_shuttle->sig.set(this, frame_shuttle_callback);
  _p->frame_shuttle->value_sig.set(this, frame_shuttle_value_callback);

  _p->start_button->sig.set(this, start_callback);
  _p->end_button->sig.set(this, end_callback);

  _p->prev->sig.set(this, prev_callback);
  _p->next->sig.set(this, next_callback);
  
  // In/out callbacks.
  
  _p->in->sig.set(this, in_callback);
  _p->in_mark->sig.set(this, in_mark_callback);
  _p->in_reset->sig.set(this, in_reset_callback);
  
  _p->out->sig.set(this, out_callback);
  _p->out_mark->sig.set(this, out_mark_callback);
  _p->out_reset->sig.set(this, out_reset_callback);
  
  // Speed callbacks.
  
  _p->speed->sig.set(this, speed_callback);
  _p->every_frame->sig.set(this, every_frame_callback);
  
  // Callbacks.
  
  prefs()->every_frame_sig.set(this, every_frame_callback);
}

Group::~Group()
{
  global()->timer_del(this);
}

void Group::menu_update(djv_ui::menu::Group * in)
{
  using namespace djv_ui::menu;
  
  DJV_DBG_OBJ("Group::menu_update");
  
  const List<djv_ui::shortcut::Shortcut> & shortcut =
    shortcut::prefs()->shortcut();
  
  in->add(menu_title, 0, 0, 0, SUB_MENU);
  
  in->add(
    menu_stop, shortcut[shortcut::PLAYBACK_STOP].value,
    _stop_callback, this,
    RADIO, STOP == _playback);
  
  in->add(
    menu_forward, shortcut[shortcut::PLAYBACK_FORWARD].value,
    _forward_callback, this,
    RADIO, FORWARD == _playback);
  
  in->add(
    menu_reverse, shortcut[shortcut::PLAYBACK_REVERSE].value,
    _reverse_callback, this,
    RADIO | DIVIDER, REVERSE == _playback);

  in->add(
    menu_start, shortcut[shortcut::PLAYBACK_START].value,
    _start_callback, this);
  
  in->add(
    menu_start_abs, shortcut[shortcut::PLAYBACK_START_ABS].value,
    _start_abs_callback, this);
  
  in->add(
    menu_end, shortcut[shortcut::PLAYBACK_END].value,
    _end_callback, this);
  
  in->add(
    menu_end_abs, shortcut[shortcut::PLAYBACK_END_ABS].value,
    _end_abs_callback, this);

  in->add(
    menu_prev, shortcut[shortcut::PLAYBACK_PREV].value,
    _prev_callback, this);
  
  in->add(
    menu_prev_x10, shortcut[shortcut::PLAYBACK_PREV_X10].value,
    _prev_x10_callback, this);
  
  in->add(
    menu_prev_x100, shortcut[shortcut::PLAYBACK_PREV_X100].value,
    _prev_x100_callback, this);

  in->add(
    menu_next, shortcut[shortcut::PLAYBACK_NEXT].value,
    _next_callback, this);
  
  in->add(
    menu_next_x10, shortcut[shortcut::PLAYBACK_NEXT_X10].value,
    _next_x10_callback, this);
  
  in->add(
    menu_next_x100, shortcut[shortcut::PLAYBACK_NEXT_X100].value,
    _next_x100_callback, this,
    DIVIDER);
  
  in->add(
    menu_in_mark, shortcut[shortcut::PLAYBACK_IN_MARK].value,
    _in_mark_callback, this);
  
  in->add(
    menu_in_reset, shortcut[shortcut::PLAYBACK_IN_RESET].value,
    _in_reset_callback, this);
  
  in->add(
    menu_out_mark, shortcut[shortcut::PLAYBACK_OUT_MARK].value,
    _out_mark_callback, this);
  
  in->add(
    menu_out_reset, shortcut[shortcut::PLAYBACK_OUT_RESET].value,
    _out_reset_callback, this,
    DIVIDER);

  in->add(
    menu_every_frame, 0,
    _every_frame_callback, this,
    TOGGLE, _every_frame);

  in->end();
}

void Group::playbar_update()
{
  using djv_ui::shortcut::serialize;
  
  DJV_DBG_OBJ("Group::playbar_update");
  
  const List<djv_ui::shortcut::Shortcut> & shortcut =
    shortcut::prefs()->shortcut();
  
  _p->stop->tooltip(str::Format(tooltip_stop).
    arg(serialize(shortcut[shortcut::PLAYBACK_STOP].value)));
  _p->forward->tooltip(str::Format(tooltip_forward).
    arg(serialize(shortcut[shortcut::PLAYBACK_FORWARD].value)));
  _p->reverse->tooltip(str::Format(tooltip_reverse).
    arg(serialize(shortcut[shortcut::PLAYBACK_REVERSE].value)));

  _p->playback_shuttle->tooltip(tooltip_playback_shuttle);
  
  _p->frame->tooltip(tooltip_frame);
  _p->frame_slider->tooltip(tooltip_frame_slider);
  _p->frame_shuttle->tooltip(tooltip_frame_shuttle);

  _p->start->tooltip(tooltip_start);
  _p->start_button->tooltip(str::Format(tooltip_start_button).
    arg(serialize(shortcut[shortcut::PLAYBACK_START].value)));

  _p->end->tooltip(tooltip_end);
  _p->end_button->tooltip(str::Format(tooltip_end_button).
    arg(serialize(shortcut[shortcut::PLAYBACK_END].value)));

  _p->prev->tooltip(str::Format(tooltip_prev).
    arg(serialize(shortcut[shortcut::PLAYBACK_PREV].value)));
  _p->next->tooltip(str::Format(tooltip_next).
    arg(serialize(shortcut[shortcut::PLAYBACK_NEXT].value)));

  _p->in->tooltip(tooltip_start);
  _p->in_mark->tooltip(str::Format(tooltip_in_mark).
    arg(serialize(shortcut[shortcut::PLAYBACK_IN_MARK].value)));
  _p->in_reset->tooltip(str::Format(tooltip_in_reset).
    arg(serialize(shortcut[shortcut::PLAYBACK_IN_RESET].value)));

  _p->out->tooltip(tooltip_end);
  _p->out_mark->tooltip(str::Format(tooltip_out_mark).
    arg(serialize(shortcut[shortcut::PLAYBACK_OUT_MARK].value)));
  _p->out_reset->tooltip(str::Format(tooltip_out_reset).
    arg(serialize(shortcut[shortcut::PLAYBACK_OUT_RESET].value)));

  _p->duration->tooltip(tooltip_duration);

  _p->speed->tooltip(tooltip_speed);
  _p->speed_real->tooltip(tooltip_speed_real);
  _p->every_frame->tooltip(tooltip_every_frame);
}

void Group::playback(PLAYBACK in)
{
  if (in == _playback) return;

  DJV_DBG_OBJ("Group::playback");
  DJV_DBG_PRN("in = " << in);
  
  _playback = in;
  
  playback_update();
  update_sig.emit(true);
}

void Group::playback_callback(PLAYBACK in)
{
  _playback = (in == _playback) ? STOP : in;

  playback_update();
  update_sig.emit(true);
}

void Group::stop_callback(bool) { playback_callback(STOP); }
void Group::forward_callback(bool) { playback_callback(FORWARD); }
void Group::reverse_callback(bool) { playback_callback(REVERSE); }

void Group::_stop_callback() { stop_callback(true); }
void Group::_forward_callback() { forward_callback(true); }
void Group::_reverse_callback() { reverse_callback(true); }

void Group::playback_update()
{
  DJV_DBG_OBJ("Group::playback_update");

  callbacks(false);
  
  switch (_playback)
  {
    case STOP: _p->stop->set(); break;
    case FORWARD: _p->forward->set(); break;
    case REVERSE: _p->reverse->set(); break;
  }
  
  global()->timer_del(this);

  switch (_playback)
  {
    case REVERSE:
    case FORWARD:
      _idle_init = true;
      global()->timer_add(this);
      break;
    case STOP: break;
  }

  callbacks(true);
}

namespace {

int time_max(const djv_image::Time & in)
{
  return in.list.size() ? (int(in.list.size()) - 1) : 0;
}

}

void Group::time(const djv_image::Time & time)
{
  DJV_DBG_OBJ("Group::time");
  DJV_DBG_PRN("time = " << time);
  
  _time = time;
  _in = 0;
  _out = time_max(_time);
  _speed = time.speed;
  _speed_real = 0.0;
  
  time_update();
  speed_update();
}

const djv_image::Time & Group::time() const { return _time; }

void Group::time_update()
{
  DJV_DBG_OBJ("Group::time_update");
  
  callbacks(false);
  
  const int max = time_max(_time);
  
  _p->frame->list(_time.list);
  _p->frame_slider->range(0, max);
  _p->frame_slider->in_out(_in, _out);

  if (0 == _in)
  {
    _p->start->set(_time.start());
    _p->start->show();
    
    _p->in->hide();
  }
  else
  {
    _p->in->list(_time.list);
    _p->in->set(_in);
    _p->in->show();
    
    _p->start->hide();
  }

  if (max == _out)
  {
    _p->end->set(_time.end());
    _p->end->show();
    
    _p->out->hide();
  }
  else
  {
    _p->out->list(_time.list);
    _p->out->set(_out);
    _p->out->show();
    
    _p->end->hide();
  }
  
  _p->duration->set(_out - _in + 1);
  _p->duration->text_color(
    (_in != 0 || _out != max) ? FL_SELECTION_COLOR : FL_FOREGROUND_COLOR
  );
  
  callbacks(true);
  
  _p->duration->update(); // XXX
}

void Group::speed_update()
{
  DJV_DBG_OBJ("Group::speed_update");
  
  callbacks(false);
  
  _p->frame->speed(_speed);
  _p->start->speed(_speed);
  _p->end->speed(_speed);
  _p->in->speed(_speed);
  _p->out->speed(_speed);
  _p->duration->speed(_speed);
  _p->speed->set(_speed);
  _p->speed->set_default(_time.speed);
  _p->speed_real->set(_speed_real);
  _p->speed_real->dropped_frames(_dropped_frames);
  _p->every_frame->set(_every_frame);
  
  callbacks(true);
}

void Group::playback_shuttle_callback(bool in)
{
  if (in)
  {
    playback(STOP);
    
    _shuttle = true;
    _shuttle_speed = 0.0;
    _idle_init = true;

    global()->timer_del(this);
    global()->timer_add(this);
  }
  else
  {
    _shuttle = false;
    _dropped_frames = false;
  
    global()->timer_del(this);
    
    _p->speed->set(_speed);
  }
}

void Group::playback_shuttle_value_callback(int in)
{
  _shuttle_speed =
    math::pow(double(math::abs(in)), 1.5) * (in >= 0 ? 1.0 : -1.0);
  //_shuttle_speed = double(in);
  
  _idle_frame = uint64_t(_idle_timer.seconds() * _shuttle_speed);
}

void Group::frame(int in, bool wrap)
{
  const int max = time_max(_time);
  
  const int tmp =
    wrap ?
    math::wrap(in, math::max(0, _in), math::min(max, _out)) :
    math::wrap(in, 0, max);
  if (tmp == _frame) return;

  DJV_DBG_OBJ("Group::frame");
  DJV_DBG_PRN("in = " << in);
  
  _frame = tmp;
  
  frame_update();
  image_sig.emit(true);
}

int Group::frame() const { return _frame; }

void Group::frame_callback(int in)
{
  playback(STOP);
  
  frame(in);
}

void Group::frame_slider_callback(int in)
{
  playback(STOP);
  
  frame(in, false);
}

void Group::frame_shuttle_callback(bool in)
{
  playback(STOP);

  _frame_tmp = _frame;
}

void Group::frame_shuttle_value_callback(int in)
{
  frame(_frame_tmp + in);
}

void Group::start_callback(bool) { frame_callback(_in); }
void Group::_start_callback() { start_callback(true); }

void Group::start_abs_callback(bool) { frame_slider_callback(0); }
void Group::_start_abs_callback() { start_abs_callback(true); }

void Group::end_callback(bool) { frame_callback(_out); }
void Group::_end_callback() { end_callback(true); }

void Group::end_abs_callback(bool) { frame_slider_callback(time_max(_time)); }
void Group::_end_abs_callback() { end_abs_callback(true); }

void Group::prev_callback(bool) { frame_callback(_frame - 1); }
void Group::_prev_callback() { prev_callback(true); }
void Group::_prev_x10_callback() { frame_callback(_frame - 10); }
void Group::_prev_x100_callback() { frame_callback(_frame - 100); }

void Group::next_callback(bool) { frame_callback(_frame + 1); }
void Group::_next_callback() { next_callback(true); }
void Group::_next_x10_callback() { frame_callback(_frame + 10); }
void Group::_next_x100_callback() { frame_callback(_frame + 100); }

void Group::frame_update()
{
  DJV_DBG_OBJ("Group::frame_update");

  callbacks(false);
  
  _p->frame->set(_frame);
  _p->frame_slider->set(_frame);
  
  callbacks(true);
}

int Group::in() const { return _in; }

void Group::in_callback(int in)
{
  _in = in;
  
  frame(_frame);
  time_update();
  update_sig.emit(true);
}

void Group::_in_goto_callback() { frame(_in); }
void Group::in_mark_callback(bool) { in_callback(_frame); }
void Group::_in_mark_callback() { in_mark_callback(true); }
void Group::in_reset_callback(bool) { in_callback(0); }
void Group::_in_reset_callback() { in_reset_callback(true); }

int Group::out() const { return _out; }

void Group::out_callback(int in)
{
  _out = in;
  
  frame(_frame);
  time_update();
  update_sig.emit(true);
}

void Group::_out_goto_callback() { frame(_out); }
void Group::out_mark_callback(bool) { out_callback(_frame); }
void Group::_out_mark_callback() { out_mark_callback(true); }
void Group::out_reset_callback(bool) { out_callback(time_max(_time)); }
void Group::_out_reset_callback() { out_reset_callback(true); }

void Group::speed(const djv_image::time::Speed & in)
{
  if (in == _speed) return;
  
  DJV_DBG_OBJ("Group::playback_speed");
  DJV_DBG_PRN("in = " << in);
  
  _speed = in;
  _speed_real = 0.0;
  _dropped_frames = false;
  
  playback_update();
  speed_update();
  overlay_sig.emit(true);
  update_sig.emit(true);
}

const djv_image::time::Speed & Group::speed() const { return _speed; }

double Group::speed_real() const { return _speed_real; }

bool Group::dropped_frames() const { return _dropped_frames; }

void Group::speed_callback(const djv_image::time::Speed & in) { speed(in); }

void Group::every_frame_callback(bool in)
{
  DJV_DBG_OBJ("Group::playback_every_frame_callback");
  DJV_DBG_PRN("in = " << in);
  
  _every_frame = in;
  
  speed_update();
  overlay_sig.emit(true);
  update_sig.emit(true);
}

void Group::_every_frame_callback() { every_frame_callback(_p->menu->value()); }

void Group::idle()
{
  DJV_DBG_OBJ("Group::idle");
  DJV_DBG_PRN("playback = " << _playback);
  DJV_DBG_PRN("frame = " << _frame);
  DJV_DBG_PRN("every frame = " << _every_frame);

  if (_idle_init)
  {
    DJV_DBG_PRN("init");

    _idle_timer.start();
    _idle_frame = 0;
    _speed_timer = _idle_timer;
    _speed_counter = 0;
    _dropped_frames = false;
    _dropped_frames_tmp = false;
    _idle_init = false;
  }
  else
  {
    _idle_timer.check();
  }
  
  const double speed =
    _shuttle ? _shuttle_speed : djv_image::time::speed_to_float(_speed);
  DJV_DBG_PRN("speed = " << speed);

  // XXX Overflow?

  const uint64_t absolute_frame = uint64_t(_idle_timer.seconds() * speed);
  int inc = int(absolute_frame - _idle_frame);
  DJV_DBG_PRN("absolute frame = " << uint(absolute_frame));
  DJV_DBG_PRN("inc = " << inc);
  if (! inc) return;

  // Calculate current frame.

  _idle_frame = absolute_frame;
  if (REVERSE == _playback) inc = -inc;
  const bool every_frame = _every_frame && ! _shuttle;
  if (every_frame) inc = inc > 0 ? 1 : (inc < 0 ? -1 : 0);
  else _dropped_frames_tmp |= inc > 1 || inc < -1;
  _speed_counter += abs(inc);
  
  // Calculate real playback speed.

  _speed_timer.check();
  if (_speed_timer.seconds() >= 1.0)
  {
    _speed_real = _speed_counter / _speed_timer.seconds();
    _speed_timer.start();

    _dropped_frames = _dropped_frames_tmp;
    _dropped_frames_tmp = false;
    _speed_counter = 0;

    _p->speed_real->set(_speed_real);
    _p->speed_real->dropped_frames(every_frame ? false : _dropped_frames);
  }

  frame(_frame + inc);
}

void Group::cache(const List<int> & in) { _p->frame_slider->cache(in); }

}
}

