/*------------------------------------------------------------------------------
 bin/djv_view/window.h
 
 Window
 ------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_WINDOW_H
#define DJV_VIEW_WINDOW_H

#include "file.h"
#include "image.h"
#include "playback.h"
#include "view.h"
#include <djv_ui/window.h>

namespace djv_view {
namespace window {
using namespace djv_base;

//------------------------------------------------------------------------------
// Options

enum RESIZE_MAX
{
  RESIZE_MAX_NONE,
  RESIZE_MAX_25,
  RESIZE_MAX_50,
  RESIZE_MAX_75
};

//------------------------------------------------------------------------------
// Main Window

class Widget : public djv_ui::window::Widget
{
public:

  Widget(const Widget * copy = 0);
  ~Widget();
  void del();

  // File.
  
  void file_open(const File &);
  void file_layer(int);
  void file_proxy(djv_image::data::PROXY);
  void file_cache(bool);
  
  // Window.
  
  void window_fullscreen(bool);
  void window_toolbar(bool);
  void window_playbar(bool);
  
  // View.

  void view_grid(view::GRID);
  void view_hud(bool);

  // Playback.

  void playback(playback::PLAYBACK);
  void playback_frame(int);
  void playback_speed(const djv_image::time::Speed &);

  // Widget.
  
  int handle(int);
  
  // Layout.

  void dirty();
  void geom(const Box2i &);
  const Box2i & geom() const;

private:

  void window_fit();
  void window_menu_update(djv_ui::menu::Group *);
  void window_toolbar_update();

  djv_image::gl_data::Option image_option() const;

  DJV_CALLBACK(Widget, file_open_callback, const File &);
  DJV_CALLBACK(Widget, file_reload_callback, bool);
  DJV_CALLBACK(Widget, file_save_callback, const File &);
  DJV_CALLBACK(Widget, file_save_frame_callback, const File &);
  DJV_CALLBACK(Widget, window_new_callback, bool);
  DJV_CALLBACK(Widget, window_copy_callback, bool);
  DJV_CALLBACK(Widget, window_close_callback, bool);
  DJV_CALLBACK(Widget, window_close2_callback, bool);
  DJV_CALLBACK(Widget, window_fit_callback, bool);
  DJV_CALLBACK(Widget, window_fullscreen_callback, bool);
  DJV_CALLBACK(Widget, window_toolbar_callback, bool);
  DJV_CALLBACK(Widget, window_playbar_callback, bool);
  DJV_FL_WIDGET_CALLBACK(Widget, _window_new_callback);
  DJV_FL_WIDGET_CALLBACK(Widget, _window_copy_callback);
  DJV_FL_WIDGET_CALLBACK(Widget, _window_close_callback);
  DJV_FL_WIDGET_CALLBACK(Widget, _window_fit_callback);
  DJV_FL_WIDGET_CALLBACK(Widget, _window_fullscreen_callback);
  DJV_FL_WIDGET_CALLBACK(Widget, _window_toolbar_callback);
  DJV_FL_WIDGET_CALLBACK(Widget, _window_playbar_callback);
  DJV_CALLBACK(Widget, view_callback, bool);
  DJV_CALLBACK(Widget, view_move_callback, const Vec2i &);
  DJV_CALLBACK(Widget, view_center_callback, bool);
  DJV_CALLBACK(Widget, view_zoom_callback, double);
  DJV_CALLBACK(Widget, view_zoom_reset_callback, bool);
  DJV_CALLBACK(Widget, view_reset_callback, bool);
  DJV_CALLBACK(Widget, view_fit_callback, bool);
  DJV_CALLBACK(Widget, view_grid_callback, view::GRID);
  DJV_CALLBACK(Widget, view_hud_callback, bool);
  DJV_CALLBACK(Widget, view_pick_callback, const Vec2i &);
  DJV_CALLBACK(Widget, view_overlay_callback, bool);
  DJV_CALLBACK(Widget, image_callback, bool);
  DJV_CALLBACK(Widget, image_redraw_callback, bool);
  DJV_CALLBACK(Widget, image_resize_callback, bool);
  DJV_CALLBACK(Widget, image_frame_store_callback, bool);
  DJV_CALLBACK(Widget, image_display_callback, bool);
  DJV_CALLBACK(Widget, image_display_value_callback, const image::Display &);
  DJV_CALLBACK(Widget, image_histogram_callback, bool);
  DJV_CALLBACK(Widget, image_info_callback, bool);
  DJV_CALLBACK(Widget, image_magnify_callback, bool);
  DJV_CALLBACK(Widget, image_sample_callback, bool);
  DJV_CALLBACK(Widget, playback_frame_shuttle_callback, bool);
  DJV_CALLBACK(Widget, playback_frame_shuttle_value_callback, int);
  DJV_CALLBACK(Widget, group_callback, bool);
  
  void file_update();
  void window_controls_update();
  void window_fullscreen_update();
  void window_resize_update();
  void view_update();
  void view_resize_update();
  void view_overlay_update();
  void image_update();
  void image_redraw_update();
  void image_dialog_update();
  void image_pick_update();
  void group_update();

  std::auto_ptr<file::Group> _file;
  bool _window_fullscreen;
  bool _window_toolbar, _window_playbar;
  std::auto_ptr<view::Group> _view;
  std::auto_ptr<image::Group> _image;
  const djv_image::Image * _image_p;
  djv_image::Image _image_tmp;
  Vec2i _image_pick;
  std::auto_ptr<playback::Group> _playback;
  int _playback_frame_tmp;
  
  struct _Widget;
  std::auto_ptr<_Widget> _p;
  
  bool _valid;
};

//------------------------------------------------------------------------------
// Preferences

class Prefs : public djv_ui::Callback_Base
{
public:

  Prefs();
  ~Prefs();
  
  // Geometry.
  
  void geom(const Box2i &);
  const Box2i & geom() const;

  // Window resize.
  
  void resize_fit(bool);
  void resize_max(RESIZE_MAX);
  bool resize_fit() const;
  RESIZE_MAX resize_max() const;

  // Window controls.
  
  void toolbar(bool);
  void playbar(bool);
  bool toolbar() const;
  bool playbar() const;
  djv_ui::Signal<bool> toolbar_sig, playbar_sig;
  
private:

  Box2i _geom;
  bool _resize_fit;
  RESIZE_MAX _resize_max;
  bool _toolbar, _playbar;
};

//------------------------------------------------------------------------------
// Preferences Widget

class Prefs_Widget : public djv_ui::widget::Widget
{
public:

  Prefs_Widget();

private:

  DJV_CALLBACK(Prefs_Widget, resize_fit_callback, bool);
  DJV_CALLBACK(Prefs_Widget, resize_max_callback, int);
  DJV_CALLBACK(Prefs_Widget, toolbar_callback, bool);
  DJV_CALLBACK(Prefs_Widget, playbar_callback, bool);
};

//------------------------------------------------------------------------------
// Window Factory

class Factory
{
public:

  ~Factory();

  Widget * add(const Widget * copy = 0);
  void del(Widget *);
  const List<Widget *> list() const;

private:
  
  List<Widget *> _list;
};

//------------------------------------------------------------------------------
// Functions

double resize_max(RESIZE_MAX);

//------------------------------------------------------------------------------
// Conversion

Str & operator >> (Str &, RESIZE_MAX &) throw (Str);
Str & operator << (Str &, RESIZE_MAX);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_resize_max();

extern const Str
  menu_title,
  menu_new,
  menu_copy,
  menu_close,
  menu_fit,
  menu_fullscreen,
  menu_toolbar,
  menu_playbar;

extern const Str
  tooltip_new,
  tooltip_copy,
  tooltip_close,
  tooltip_fit,
  tooltip_fullscreen;

extern const Str
  prefs_title,
  prefs_resize_group,
  prefs_resize_fit,
  prefs_resize_max,
  prefs_controls_group,
  prefs_toolbar,
  prefs_playbar;

//------------------------------------------------------------------------------
// Global

Prefs * prefs();
Factory * factory();

}
}

#endif

