/*------------------------------------------------------------------------------
 include/djv_base/dbg.h
 
 Debugging
 ---------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_DBG_H
#define DJV_DBG_H

#include <djv_base/list.h>
#include <djv_base/str.h>
#include <djv_base/time.h>

namespace djv_base {
namespace dbg {

//------------------------------------------------------------------------------
// Assertion

#if defined(DJV_ASSERT)
#undef DJV_ASSERT
#define DJV_ASSERT(IN) if (! (IN)) dbg::_djv_assert(__FILE__, __LINE__)
#else
#define DJV_ASSERT(IN)
#endif

void _djv_assert(const char * file, int line);

//------------------------------------------------------------------------------
// Debugging Messages

enum LINE
{
  LINE_BEGIN,
  LINE_END
};

class Dbg
{
public:

  Dbg(const Str &);
  Dbg(const Str & prefix, const Str &);
  ~Dbg();

  void add(const Str &);

  Dbg & operator << (LINE);

private:

  void init(const Str &);
  
  Str _tmp;
  time::Timer _timer;
};

Dbg & operator << (Dbg &, const Str &);
Dbg & operator << (Dbg &, const char *);
Dbg & operator << (Dbg &, bool);
Dbg & operator << (Dbg &, int);
Dbg & operator << (Dbg &, uint);
Dbg & operator << (Dbg &, double);
Dbg & operator << (Dbg &, const List<Str> &);
template<class T>
inline Dbg & operator << (Dbg &, const List<T> &);

#if defined(DJV_DBG)
#define DJV_DBG_OBJ(in) \
  djv_base::dbg::Dbg _dbg(__FILE__, in)
#define DJV_DBG_PRN(in) \
  _dbg << djv_base::dbg::LINE_BEGIN << in << djv_base::dbg::LINE_END
#else
#define DJV_DBG_OBJ(in)
#define DJV_DBG_PRN(in)
#endif

//------------------------------------------------------------------------------
// Conversion

Str u8(uint8_t);
Str u16(uint16_t);
Str u32(uint32_t);

}

//------------------------------------------------------------------------------
// Declarations

using dbg::Dbg;

}

#include <djv_base/dbg_.h>

#endif

