/*------------------------------------------------------------------------------
 include/djv_base/file_info_.h
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_base/math.h>

namespace djv_base {
namespace file {

inline int Seq::start() const
{
  return list.size() ? list[0] : 0;
}

inline int Seq::end() const
{
  return list.size() ? list[list.size() - 1] : 0;
}

inline File::operator Str() const { return get(); }

inline bool File::seq_add(const File & in)
{
  // Compare.
  
  if (extension != in.extension) return false;
  if (base != in.base) return false;
  if (! in.seq_valid()) return false;
  //if (seq_valid() && seq.pad != in.seq.pad) return false;

  // Update sequence.
  
  seq.list += in.seq.list;
  seq.pad = math::max(seq.pad, in.seq.pad);

  // Update information.
  
  size += in.size;
  user = math::max(user, in.user);
  time = math::max(time, in.time);

  return true;
}

inline bool File::seq_valid() const
{
  return type != DIR && number.size();
  //return type != DIR && seq.list.size();
}

inline bool File::seq_wildcard() const
{
  return type != DIR && number.size() && '#' == number[0];
}

inline bool operator == (const Seq & a, const Seq & b)
{
  return a.list == b.list && a.pad == b.pad;
}

inline bool operator != (const Seq & a, const Seq & b)
{
  return ! (a == b);
}

inline bool operator == (const File & a, const File & b)
{
  return a.get() == b.get();
}

inline bool operator != (const File & a, const File & b)
{
  return ! (a == b);
}

inline int str_to_frame(const Str & in, int * pad)
{
  const uint size = in.size();
  if (size)
    if ('#' == in[0]) return -1;

  if (pad)
  {
    *pad = 0;
    if (size > 1)
      if ('0' == in[0]) *pad = size;
  }
  return str::str_to_int(in);
}

}
}

