/*------------------------------------------------------------------------------
 include/djv_image/app.h
 
 Command Line Image Application
 ------------------------------
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_APP_H
#define DJV_IMAGE_APP_H

#include <djv_image/gl.h>
#include <djv_image/gl_offscreen.h>
#include <djv_image/time.h>
#include <djv_image/io.h>
#include <djv_base/app.h>

namespace djv_image {
namespace app {

//------------------------------------------------------------------------------
// Application

class App : public djv_base::app::App
{
public:

  App(const Str & name, int & argc, char ** argv) throw (Error);
  virtual ~App();

  // Global.
  
  gl::Global * gl() const;
  gl_data::Global * gl_data() const;
  gl_offscreen::Global * gl_offscreen() const;
  time::Global * time() const;
  io::Global * io() const;

  // Information.
  
  virtual Str info() const;

  // Command line.
  
  void cmdln(Str &) throw (Error);
  virtual Str cmdln_help() const;

private:

  std::auto_ptr<gl::Global> _gl;
  std::auto_ptr<gl_data::Global> _gl_data;
  std::auto_ptr<gl_offscreen::Global> _gl_offscreen;
  std::auto_ptr<time::Global> _time;
  std::auto_ptr<io::Global> _io;
};

//------------------------------------------------------------------------------
// Labels

extern const Str
  label_info,
  label_cmdln_help;

//------------------------------------------------------------------------------
// Global

App * global();

}
}

#endif

