/*------------------------------------------------------------------------------
 include/djv_image/io_sgi.h

 Image I/O - SGI
 ---------------
 
 Silicon Graphics image file format.
 
 Support:
 
 * Image data: L, LA, RGB, RGBA, 8-bit, 16-bit.
 * File compression.
 
 References:
 
 * Paul Haeberli, "The SGI Image File Format, Version 1.00"

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_IO_SGI_H
#define DJV_IMAGE_IO_SGI_H

#include <djv_image/io.h>

namespace djv_image {
namespace io_sgi {

//------------------------------------------------------------------------------
// Information

static const Str name = "SGI";

static const List<Str> extension_list = List<Str>() <<
  ".sgi" <<
  ".rgba" <<
  ".rgb" <<
  ".bw";

//------------------------------------------------------------------------------
// Options

enum COMPRESS
{
  COMPRESS_NONE,
  COMPRESS_RLE
};

enum SAVE_OPTION
{
  SAVE_COMPRESS
};

struct Save_Option
{
  Save_Option() :
    compress(COMPRESS_NONE)
  {}

  COMPRESS compress;
};

//------------------------------------------------------------------------------
// Load Plugin

class Load : public io::Load
{
public:

  Load();

  Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  void open(const File &, Info *, Tag *, Time *) throw (Error);
  void open(const File &, List<Info> *, Tag *, Time *) throw (Error);
  const Image * get(int frame, int layer, data::PROXY) throw (Error);

private:

  void _open(const File &, Info *) throw (Error);

  File _file;
  djv_base::Io _io;
  bool _compress;
  mem::Buffer<uint32_t> _rle_offset, _rle_size;
  Image _image;
  Data _tmp;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public io::Save
{
public:

  Save();
  
  Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;

  bool option(const Str &, Str *);
  Str option(const Str &) const;
  const List<Str> & option_list() const;

  void open(const File &, const Info &, const Tag &, const Time &)
    throw (Error);
  void set(const Image &, int frame) throw (Error);

private:

  void _open(const File &) throw (Error);

  Save_Option _option;
  File _file;
  djv_base::Io _io;
  mem::Buffer<uint32_t> _rle_offset, _rle_size;
  Info _info;
  Image _image;
  Data _tmp;
};

//------------------------------------------------------------------------------
// Functions

void info_load(djv_base::io::Io *, Info *, bool * compress) throw (Error);
void info_save(djv_base::io::Io *, const Info &, bool compress) throw (Error);

bool rle_load(
  const uint8_t * in,
  const uint8_t * end,
  void * out,
  int size,
  int bytes
);

size_t rle_save(
  const void * in,
  void * out,
  int size,
  int bytes
);

//------------------------------------------------------------------------------
// Conversion

Str & operator >> (Str &, COMPRESS &) throw (Str);
Str & operator << (Str &, COMPRESS);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_compress();
const List<Str> & label_save_option();

}
}

#endif
