/*------------------------------------------------------------------------------
 include/djv_image/profile.h
 
 Color Profiles
 --------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_PROFILE_H
#define DJV_IMAGE_PROFILE_H

#include <djv_image/color.h>
#include <djv_image/data.h>

namespace djv_image {
namespace profile {

//------------------------------------------------------------------------------
// Color Profile

struct Exposure
{
  Exposure() :
    value(0),
    defog(0),
    knee_low(0),
    knee_high(5),
    gamma(2.2)
  {}

  double value, defog, knee_low, knee_high, gamma;
};

enum PROFILE
{
  RAW,
  LUT,
  EXPOSURE
};

struct Profile
{
  Profile() :
    type(RAW)
  {}

  PROFILE type;
  Data lut;
  Exposure exposure;
};

//------------------------------------------------------------------------------
// Comparison

bool operator == (const Exposure &, const Exposure &);
bool operator != (const Exposure &, const Exposure &);

// XXX LUT data is not compared.

bool operator == (const Profile &, const Profile &);
bool operator != (const Profile &, const Profile &);

//------------------------------------------------------------------------------
// Conversion

Str & operator >> (Str &, Exposure &) throw (Str);
Str & operator >> (Str &, PROFILE &) throw (Str);
Str & operator << (Str &, const Exposure &);
Str & operator << (Str &, PROFILE);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_profile();

//------------------------------------------------------------------------------
// Debugging

Dbg & operator << (Dbg &, PROFILE &);

}

//------------------------------------------------------------------------------
// Declarations

using profile::Profile;

}

#endif

