/*------------------------------------------------------------------------------
 include/djv_ui/layout.h
 
 Widget Layout
 -------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_LAYOUT_H
#define DJV_UI_LAYOUT_H

#include <djv_base/box.h>
#include <djv_ui/base.h>

namespace djv_ui {
namespace layout {

//------------------------------------------------------------------------------
// Layout Item

enum ITEM
{
  WIDGET,
  LAYOUT,
  SPACER
};

enum SIZE
{
  FIXED,
  STRETCH
};

enum ALIGN
{
  LEFT,
  CENTER,
  RIGHT
};

class Item
{
public:

  Item(ITEM);
  virtual ~Item() = 0;
  
  // Value.
  
  inline ITEM item() const;

  // Layout.
  
  virtual void dirty();
  virtual void update();
  virtual void geom(const Box2i &);
  inline virtual const Box2i & geom() const;
  inline virtual const Vec2i & size_hint() const;
  inline virtual const Vec2i & size_min() const;
  inline virtual int visible() const;

  // Size.

  virtual void size(SIZE, SIZE);
  virtual void align(ALIGN, ALIGN);
  inline const SIZE * size() const;
  inline const ALIGN * align() const;
  
  // XXX Cheesy way to align items.

  void group(const List<Item *> &);
  inline const List<Item *> & group() const;
  inline virtual Item * group_align();
  inline virtual Item * label_align();

  // Parent.

  virtual void parent(Item *);
  inline Item * parent() const;
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::layout::Item"; }

protected:

  void size_hint(const Vec2i &);

private:

  ITEM _item;
  Box2i _geom;
  Vec2i _size_hint;
  SIZE _size [2];
  ALIGN _align [2];
  List<Item *> _group;
  Item * _parent;
};

//------------------------------------------------------------------------------
// Layout Spacer

class Spacer : public Item
{
public:

  Spacer();
};

//------------------------------------------------------------------------------
// Layout

class Layout : public Item
{
public:

  Layout(Item * parent = 0);
  virtual ~Layout() = 0;

  // Items.
  
  virtual void add(Item *);
  virtual void insert(Item *, int);
  virtual Item * remove(int);
  virtual Item * remove(Item *);
  virtual void del(int);
  virtual void del(Item *);
  virtual void del();
  inline const List<Item *> & list() const;

  // Layout.

  void spacing(int);
  void margin(int);
  int spacing() const;
  int margin() const;

  virtual void dirty();
  virtual int visible() const;
  
  // Debugging.
  
  void dbg(int indent = 0);
  virtual Str dbg_name() const { return "djv::ui::layout::Layout"; }

private:

  List<Item *> _list;
  int _spacing, _margin;
};

//------------------------------------------------------------------------------
// Functions

void group(const List<Item *> &);

}
}

#include <djv_ui/layout_.h>

#endif

