/*------------------------------------------------------------------------------
 include/djv_ui/float.h
 
 Floating Point Number Widgets
 -----------------------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_FLOAT_H
#define DJV_UI_FLOAT_H

#include <djv_ui/num.h>

namespace djv_ui {
namespace num {

//------------------------------------------------------------------------------
// Float Edit

class Float_Edit : public widget::Widget, public Float_Base
{
public:

  Float_Edit();
  virtual ~Float_Edit();
  
  // Value.
  
  Signal<double> sig;
  
  // Widget.
  
  virtual int handle(int);
  
  // Layout.
  
  virtual Item * group_align();

private:
  
  void value_callback();
  DJV_CALLBACK(Float_Edit, widget_callback, const Str &);
  DJV_CALLBACK(Float_Edit, inc_callback, bool);
  DJV_CALLBACK(Float_Edit, dec_callback, bool);
  
  void value_update();
  
  struct _Float_Edit;
  std::auto_ptr<_Float_Edit> _p;
};

//------------------------------------------------------------------------------
// Float Display

class Float_Display : public widget::Widget
{
public:

  Float_Display();
  virtual ~Float_Display();
  
  // Value.
  
  void set(double);

  void precision(int);
  int precision() const;
  
  // Widget.
  
  virtual void text_color(const Fl_Color &);
  virtual const Fl_Color & text_color() const;
  
  virtual void draw();
  
  // Layout.
  
  virtual void dirty();
  
private:

  int _precision;
  
  struct _Float_Display;
  std::auto_ptr<_Float_Display> _p;
};

//------------------------------------------------------------------------------
// Float Slider

class Float_Slider : public widget::Widget, public Float_Base
{
public:

  Float_Slider(
    double min = 0.0,
    double max = 1.0,
    const Str & label = Str()
  );
  virtual ~Float_Slider();
  
  // Value.
  
  virtual void default_valid(bool);
  bool default_valid() const;
  
  virtual void range(double min, double max);
  virtual void slider_range(double min, double max);
  
  Signal<double> sig;
  
  // Label.
  
  void label(widget::Widget *);
  widget::Widget * label() const;
  virtual Item * label_align();

  // Widget.

  virtual void text_font(const Fl_Font &);
  virtual void text_color(const Fl_Color &);
  virtual const Fl_Font & text_font() const;
  virtual const Fl_Color & text_color() const;
  
  virtual int handle(int);
  
  // Layout.
  
  virtual Item * group_align();
  
private:
  
  void value_callback();
  void range_callback();
  DJV_CALLBACK(Float_Slider, widget_callback, const Str &);
  DJV_CALLBACK(Float_Slider, inc_callback, bool);
  DJV_CALLBACK(Float_Slider, dec_callback, bool);
  DJV_CALLBACK(Float_Slider, slider_callback, double);
  DJV_CALLBACK(Float_Slider, default_callback, bool);
  
  void value_update();
  void range_update();
  
  struct _Float_Slider;
  std::auto_ptr<_Float_Slider> _p;
};

//------------------------------------------------------------------------------
// Labels

extern const Str format_float;

}
}

#endif

