/*------------------------------------------------------------------------------
 include/djv_ui/style.h
 
 Style
 -----
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_STYLE_H
#define DJV_UI_STYLE_H

#include <djv_image/color.h>
#include <djv_ui/widget.h>

namespace djv_ui {
namespace style {

//------------------------------------------------------------------------------
// Options

enum COLOR
{
  COLOR_DARK,
  COLOR_NEUTRAL,
  COLOR_LIGHT,
  COLOR_CUSTOM
};

struct Color
{
  Color();
  
  djv_image::Color foreground;
  djv_image::Color background;
  djv_image::Color background2;
  djv_image::Color select;
};

enum SIZE
{
  SIZE_SMALL,
  SIZE_MEDIUM,
  SIZE_LARGE
};

//------------------------------------------------------------------------------
// Style

class Style
{
public:

  Style();
  ~Style();

  // Color.

  void color(COLOR);
  void color_custom(const Color &);
  COLOR color() const;
  const Color & color_value() const;
  const Color & color_custom() const;

  // Size.

  void size(SIZE);
  SIZE size() const;
  int size_pick() const;
  int size_handle() const;

  // Font.

  Fl_Font font() const;
  Fl_Font font_bold() const;
  int font_size() const;
  
  // Layout.

  int spacing() const;
  int spacing_large() const;
  
  int margin() const;
  int margin_text() const;
  int margin_widget() const;
  int margin_large() const;

  int size_icon() const;
  int size_tool_icon() const;
  int size_button() const;
  int size_text() const;
  int size_swatch() const;
  int size_thumbnail() const;
  
private:

  class _Style;
  std::auto_ptr<_Style> _p;
};

//------------------------------------------------------------------------------
// Preferences

class Prefs
{
public:

  Prefs();
  ~Prefs();
  
  // Color.
  
  void color(COLOR);
  void color_custom(const Color &);
  COLOR color() const;
  const Color & color_custom() const;
  
  // Size.
  
  void size(SIZE);
  SIZE size() const;
  
private:

  COLOR _color;
  Color _color_custom;
  SIZE _size;
};

//------------------------------------------------------------------------------
// Comparison

bool operator == (const Color &, const Color &);
bool operator != (const Color &, const Color &);

//------------------------------------------------------------------------------
// Conversion

Str & operator >> (Str &, COLOR &) throw (Str);
Str & operator >> (Str &, Color &) throw (Str);
Str & operator >> (Str &, SIZE &) throw (Str);
Str & operator << (Str &, COLOR);
Str & operator << (Str &, const Color &);
Str & operator << (Str &, SIZE);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_color();
const List<Str> & label_size();

//------------------------------------------------------------------------------
// Global

Style * global();
Prefs * prefs();

}
}

#endif

