/*------------------------------------------------------------------------------
 lib/djv_base/str_format.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_base/dbg.h>
#include <djv_base/str.h>

namespace djv_base {
namespace str {

Format::Format(const Str & in) :
  _value(in)
{}

Format Format::arg(const Str & in, int width, char fill) const
{
  //DJV_DBG_OBJ("Format::arg");
  //DJV_DBG_PRN("value = " << _value);
  //DJV_DBG_PRN("in = " << in << " (" << in.size() << ")");
  //DJV_DBG_PRN("width = " << width);
  //DJV_DBG_PRN("fill = " << fill);
  
  // Find argument string "%%".

  int size = int(_value.size()), index = -1;
  for (int i = 0; i < size - 1; ++i)
    if ('%' == _value[i] && '%' == _value[i + 1])
    {
      index = i;
      break;
    }
  if (-1 == index) return _value;
  //DJV_DBG_PRN("index = " << index);
  
  // Format input.

  Str tmp = in;
  if (width)
  {
    int w = (width < 0 ? -width : width) - int(in.size());
    if (w < 0) w = 0;
    if (width < 0) tmp += Str(w, fill);
    else tmp.insert(tmp.begin(), w, fill);
  }
  
  // Replace.
  
  const Str out = Str(_value, 0, index) + tmp +
    Str(_value, index + 2, _value.size() - (index + 2));
  //DJV_DBG_PRN("out = " << out);
  return out;
}

Format Format::arg(const List<Str> & in, const Str & separator, int width,
  char fill) const
{
  return arg(join(in, separator), width, fill);
}

Format Format::arg(int in, int width, char fill) const
{
  return arg(int_to_str(in), width, fill);
}

Format Format::arg(uint in, int width, char fill) const
{
  return arg(int_to_str(in), width, fill);
}

Format Format::arg(double in, int width, int precision, char fill) const
{
  return arg(float_to_str(in, precision), width, fill);
}

Format::operator Str () const
{
  return _value;
}

}
}

