/*------------------------------------------------------------------------------
 lib/djv_image/gl_data.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include "gl_data_fixed.h"
#include "gl_data_program.h"

namespace djv_image {
namespace gl_data {

Filter::Filter() :
  value(global()->filter()),
  _default(global()->filter_default()),
  custom(global()->filter_custom())
{}

struct Data::_Data
{
  fixed::Fixed fixed;
  program::Program program;
};

Data::Data() :
  _p(new _Data)
{
  DJV_DBG_OBJ("Data::Data");
}

Data::~Data()
{
  DJV_DBG_OBJ("Data::~Data");
}

void Data::draw(
  const djv_image::Data & in,
  const Option & option
) const throw (Error) {

  DJV_DBG_OBJ("Data::draw");
  DJV_DBG_PRN("in = " << in);
  
  switch (gl::global()->pipeline())
  {
    case gl::FIXED: _p->fixed.draw(in, option); break;
    case gl::PROGRAM: _p->program.draw(in, option); break;
  }
}

void Data::read(djv_image::Data * out)
{
  read(out, Box2i(out->size()));
}

void Data::read(djv_image::Data * out, const Box2i & area)
{
  DJV_DBG_OBJ("Data::read");
  DJV_DBG_PRN("out = " << *out);
  DJV_DBG_PRN("area = " << area);
  
  const Info & info = out->info();
  
  DJV_DBG_GL(glPushAttrib(
    GL_CURRENT_BIT |
    GL_ENABLE_BIT |
    GL_PIXEL_MODE_BIT));
  DJV_DBG_GL(glDisable(GL_DITHER));
  
  // XXX
  
  if (GLEW_ARB_color_buffer_float)
    DJV_DBG_GL(glClampColorARB(GL_CLAMP_READ_COLOR_ARB, GL_FALSE));
  
  // XXX OpenGL sums color components for luminance?
  
  switch (pixel::format(info.pixel))
  {
    case pixel::L:
    case pixel::LA:
      DJV_DBG_GL(glPixelTransferf(GL_GREEN_SCALE, 0.0));
      DJV_DBG_GL(glPixelTransferf(GL_BLUE_SCALE, 0.0));
      break;
    default: break;
  }
  
  state_pack(info, area.position);

  DJV_DBG_GL(glReadPixels(
    0, 0, area.w, area.h,
    gl::format(info.pixel, info.bgr),
    gl::type(info.pixel),
    out->data()));

  //state_reset();

  DJV_DBG_GL(glPopAttrib());
}

}
}
