/*------------------------------------------------------------------------------
 lib/djv_image/io_dpx_load.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include <djv_image/io_dpx.h>

namespace djv_image {
namespace io_dpx {

Load::Load() :
  _print(false)
{}

io::Plugin * Load::copy() const
{
  Load * out = new Load;
  out->_option = _option;
  return out;
}

const Str & Load::name() const { return io_dpx::name; }

const List<Str> & Load::extension_list() const
  { return io_dpx::extension_list; }

bool Load::option(const Str & in, Str * data)
{
  _print_lut.reset();
  
try
{
  if (str::compare(in, option_list()[LOAD_COLOR])) *data >> _option.color;
  else if (str::compare(in, option_list()[LOAD_PRINT])) *data >> _option.print;
}
catch (Str) { return false; }
  return true;
}

Str Load::option(const Str & in) const
{
  Str out;
  if (str::compare(in, option_list()[LOAD_COLOR])) out << _option.color;
  else if (str::compare(in, option_list()[LOAD_PRINT])) out << _option.print;
  return out;
}

const List<Str> & Load::option_list() const { return label_load_option(); }

void Load::open(const File & in, Info * info, Tag * tag, Time * time)
  throw (Error)
{
  DJV_DBG_OBJ("Load::open");
  DJV_DBG_PRN("in = " << in);

  _file = in;
  _open(_file.get(_file.seq.start()), info, tag, &time->speed);
  
  if (file::SEQ == _file.type)
    time->list = _file.seq.list;
  DJV_DBG_PRN("time = " << *time);
}

void Load::open(const File & in, List<Info> * info, Tag * tag, Time * time)
  throw (Error)
{
  io::Load::open(in, info, tag, time);
}

void Load::_open(const File & in, Info * info, Tag * tag, time::Speed * speed)
  throw (Error)
{
  DJV_DBG_OBJ("Load::_open");
  DJV_DBG_PRN("in = " << in);

  _io.open(in, djv_base::io::READ);
  
  info->name = in;
  _print = false;
  info_load(&_io, info, tag, speed, &_print);
  DJV_DBG_PRN("info = " << *info);
  DJV_DBG_PRN("print = " << _print);
}

const Image * Load::get(int frame, int, data::PROXY proxy) throw (Error)
{
  DJV_DBG_OBJ("Load::get");
  DJV_DBG_PRN("frame = " << frame);
  DJV_DBG_PRN("proxy = " << proxy);

  // Open.
  
  const File file = File(_file.get(frame != -1 ? frame : _file.seq.start()));
  DJV_DBG_PRN("file = " << file);
  
  Info info;
  _open(file, &info, &_image.tag);

  if (io_cineon::COLOR_PRINT == _option.color ||
    (io_cineon::COLOR_AUTO == _option.color && _print))
  {
    DJV_DBG_PRN("profile");
    _image.profile.type = profile::LUT;
    if (! _print_lut.get())
      _print_lut = std::auto_ptr<Data>(new Data(unprint_lut(_option.print)));
    _image.profile.lut = *_print_lut;
  }
  else
  {
    _image.profile = Profile();
  }
  
  const Cache cache(file, frame, 0, proxy);
  if (cache == _cache)
  {
    DJV_DBG_PRN("cached");
    return &_image;
  }
  else
  {
    _cache = cache;
  }

  // Read.
  
  _io.readahead();
  const uint8_t * p = _io.mmap_p();
  _io.seek(data::bytes_data(info));
  
  if (! proxy)
  {
    _image.set(info, p);
  }
  else
  {
    _tmp.set(info, p);
    info.size = data::proxy_scale(info.size, proxy);
    info.proxy = proxy;
    _image.set(info);
    data::proxy_scale(_tmp, &_image, proxy);
  }
  
  DJV_DBG_PRN("image = " << _image);  
  return &_image;
}

}
}
