/*------------------------------------------------------------------------------
 lib/djv_image/io_tga_save.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include <djv_image/io_tga.h>

namespace djv_image {
namespace io_tga {

Save::Save()
{
  _io.endian(mem::endian() != mem::LSB);
}

io::Plugin * Save::copy() const
{
  Save * out = new Save;
  out->_option = _option;
  return out;
}

const Str & Save::name() const { return io_tga::name; }

const List<Str> & Save::extension_list() const
  { return io_tga::extension_list; }

bool Save::option(const Str & in, Str * data)
{
try
{
  if (str::compare(in, option_list()[SAVE_COMPRESS])) *data >> _option.compress;
}
catch (Str) { return false; }
  return true;
}

Str Save::option(const Str & in) const
{
  Str out;
  if (str::compare(in, option_list()[SAVE_COMPRESS])) out << _option.compress;
  return out;
}

const List<Str> & Save::option_list() const { return label_save_option(); }

void Save::open(const File & in, const Info & info, const Tag &,
  const Time & time) throw (Error)
{
  DJV_DBG_OBJ("Save::open");
  DJV_DBG_PRN("in = " << in);
  
  _file = in;
  if (time.list.size() > 1) _file.type = file::SEQ;
  
  _info = Info();
  _info.size = info.size;
  _info.pixel = pixel::pixel(pixel::format(info.pixel), pixel::U8);
  switch (pixel::format(_info.pixel))
  {
    case pixel::RGB:
    case pixel::RGBA: _info.bgr = true; break;
    default: break;
  }
  _info.endian = mem::LSB;
  DJV_DBG_PRN("info = " << _info);
  
  _image.set(_info);
}

void Save::_open(const File & in) throw (Error)
{
  DJV_DBG_OBJ("Save::_open");
  DJV_DBG_PRN("in = " << in);

  _io.open(in, djv_base::io::WRITE);
  
  info_save(&_io, _info, _option.compress);
}

void Save::set(const Image & in, int frame) throw (Error)
{
  DJV_DBG_OBJ("Save::set");
  DJV_DBG_PRN("in = " << in);

  // Open.
  
  _open(_file.get(frame));
  
  // Write.
  
  const Data * p = &in;
  if (in.info() != _info)
  {
    DJV_DBG_PRN("convert = " << _image);
    _image.zero();
    gl_data::copy(in, &_image);
    p = &_image;
  }
  
  if (! _option.compress)
  {
    _io.set(p->data(), p->bytes_data());
  }
  else
  {
    const int w = p->w(), h = p->h();
    const int channels = pixel::channels(p->info().pixel);
    mem::Buffer<uint8_t> scanline(w * channels * 2);

    for (int y = 0; y < h; ++y)
    {
      const size_t size = rle_save(p->data(0, y), scanline(), w, channels);
      _io.set(scanline.data(), size);
    }
  }

  _io.close();
}

}
}
