/*------------------------------------------------------------------------------
 lib/djv_ui/choice_edit.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/choice.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/menu.h>
#include <djv_ui/style.h>
#include <djv_ui/text.h>
#include <FL/Fl.H>
#include <FL/fl_draw.H>

namespace djv_ui {
namespace choice {

struct Edit::_Edit
{
  text::Edit * widget;
  menu::Popup * menu;
};

Edit::Edit(const List<Str> & list) :
  sig(this),
  _list(list),
  _p(new _Edit)
{
  frame(FL_DOWN_FRAME);
  
  // Widgets.
  
  _p->widget = new text::Edit;
  _p->widget->frame(FL_NO_BOX);
  
  _p->menu = new menu::Popup;

  // Layout.

  size(layout::FIXED, layout::FIXED);
  align(layout::LEFT, layout::CENTER);
  
  Layout_H * layout = new Layout_H(this);
  layout->spacing(2);
  layout->margin(0);
  layout->add(_p->widget);
  layout->stretch(_p->widget);
  layout->add(_p->menu);
  
  // Initialize.

  list_update();

  // Callbacks.

  _p->widget->sig.set(this, widget_callback);
  _p->menu->sig.set(this, menu_callback);
}

Edit::~Edit()
{}

void Edit::set(const Str & in)
{
  _p->widget->set(in);
}
  
Str Edit::get() const { return _p->widget->get(); }

void Edit::list(const List<Str> & in)
{
  if (in == _list) return;
  
  _list = in;
  
  list_update();
}

const List<Str> & Edit::list() const { return _list; }

void Edit::size_str(const Str & in)
{
  _p->widget->size_str(in);
}

void Edit::widget_callback(const Str & in)
{
  sig.emit(in);
}

void Edit::menu_callback(int in)
{
  set(_list[in]);
}

void Edit::list_update()
{
  _p->menu->set(_list);
  
  // Set to width of largest menu item.
  
  if (_list.size() && ! _p->widget->size_str().size())
  {
    fl_font(text_font(), style::global()->font_size());
    
    uint i = 0;
    for (uint j = 1; j < _list.size(); ++j)
      if (fl_width(_list[j].c_str()) > fl_width(_list[i].c_str()))
        i = j;
    _p->widget->size_str(_list[i] + " ");
    
    update();
  }
}

}
}
