/*------------------------------------------------------------------------------
 lib/djv_ui/dialog_fnc.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/dialog.h>

namespace djv_ui {
namespace dialog {

const Str
  label_message = "Message",
  label_question = "Question",
  label_input = "Input",
  label_choice = "Choice",
  label_progress = "Progress",
  label_progress_estimate = "Estimated: %% (%% Frames/Second)",
  label_progress_elapsed = "Elapsed: %%",
  label_color = "Color",
  label_ok = "&Ok",
  label_close = "Clos&e",
  label_cancel = "&Cancel",
  label_yes = "&Yes",
  label_no = "&No",
  label_show = "Show",
  label_pin = "Pin";

Global::Global() :
  _message(new Message),
  _question(new Question),
  _input(new Input),
  _choice(new Choice),
  _progress(new Progress),
  _color(new Color)
{}

Global::~Global()
{}

void Global::message(const Str & label) const
{
  _message->label(label);
  _message->show();
}

void Global::question(
  const Str & label,
  Callback_Base * in,
  Signal<bool>::Callback * callback
) const {
  _question->label(label);
  _question->sig.del();
  _question->sig.set(in, callback);
  _question->show();
}

void Global::input(
  const Str & label,
  const Str & value,
  Callback_Base * in,
  Signal<const Str &>::Callback * callback
) const {
  _input->set(value);
  _input->label(label);
  _input->sig.del();
  _input->sig.set(in, callback);
  _input->show();
}

void Global::choice(
  const Str & label,
  const List<Str> & list,
  Callback_Base * in,
  Signal<int>::Callback * callback
) const {
  _choice->list(list);
  _choice->label(label);
  _choice->sig.del();
  _choice->sig.set(in, callback);
  _choice->show();
}

void Global::progress(
  const Str & label,
  int value,
  Callback_Base * in,
  Signal<int>::Callback * callback,
  Signal<bool>::Callback * del_callback
) const {
  _progress->sig.del();
  _progress->sig.set(in, callback);
  _progress->del_sig.del();
  _progress->del_sig.set(in, del_callback);
  _progress->set(value);
  _progress->label(label);
  _progress->show();
}

void Global::progress_del(Callback_Base * in) const
{
  _progress->sig.del(in);
  _progress->hide();
}

void Global::color(
  const djv_image::Color & value,
  Callback_Base * in,
  Signal<const djv_image::Color &>::Callback * callback
) const {
  _color->sig.del();
  _color->sig.set(in, callback);
  _color->set(value);
  _color->show();
}

}
}
