/*------------------------------------------------------------------------------
 lib/djv_ui/dialog_input.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/button.h>
#include <djv_ui/dialog.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/label.h>
#include <djv_ui/style.h>
#include <djv_ui/text.h>

namespace djv_ui {
namespace dialog {

struct Input::_Input
{
  text::Edit * widget;
  label::Widget * label;
};

Input::Input(const Str & label, const Str & value) :
  Message_Base(label_input),
  sig(this),
  _p(new _Input)
{
  // Widgets.

  _p->widget = new text::Edit(value);
  _p->label = new label::Widget(label);

  button::Push * ok_button = new button::Push(label_ok);
  button::Push * close_button = new button::Push(label_close);

  // Layout.

  Layout_V * layout = new Layout_V(this);
  
  Layout_H * layout_h = new Layout_H(layout);
  layout->stretch(layout_h);
  layout_h->margin(style::global()->margin_large());
  layout_h->add(_p->label);
  layout_h->add(_p->widget);
  layout_h->stretch(_p->widget);
  
  layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add_stretch();
  layout_h->add(ok_button);
  layout_h->add(close_button);
  
  // Initialize.

  size(size_hint());

  // Callbacks.
  
  _p->widget->action_sig.set(this, widget_callback);
  ok_button->sig.set(this, ok_callback);
  close_button->sig.set(this, close_callback);
}

Input::~Input()
{}

void Input::set(const Str & in)
{
  _p->widget->set(in);
  
  size(size_hint());
}

Str Input::get() const { return _p->widget->get(); }

void Input::label(const Str & in)
{
  _p->label->set(in);
  
  size(size_hint());
}

const Str & Input::label() const { return _p->label->get(); }

void Input::show()
{
  _p->widget->take_focus();

  Message_Base::show();
}

void Input::widget_callback(const Str &)
{
  ok_callback(true);
}

void Input::ok_callback(bool)
{
  sig.emit(_p->widget->get());
  
  hide();
}

void Input::close_callback(bool)
{
  hide();
}

}
}

