/*------------------------------------------------------------------------------
 lib/djv_ui/dialog_question.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/button.h>
#include <djv_ui/dialog.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/label.h>
#include <djv_ui/style.h>

namespace djv_ui {
namespace dialog {

struct Question::_Question
{
  label::Widget * widget;
  button::Push * ok, * close;
};

Question::Question(const Str & label) :
  Message_Base(label_question),
  sig(this),
  _p(new _Question)
{
  // Widgets.

  _p->widget = new label::Widget(label);

  _p->ok = new button::Push(label_yes);
  _p->close = new button::Push(label_no);

  // Layout.
  
  Layout_V * layout = new Layout_V(this);
  
  Layout_H * layout_h = new Layout_H(layout);
  layout->stretch(layout_h);
  layout_h->margin(style::global()->margin_large());
  layout_h->add_stretch(false);
  layout_h->add(_p->widget);
  layout_h->add_stretch(false);
  
  layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add_stretch();
  layout_h->add(_p->ok);
  layout_h->add(_p->close);
  
  // Initialize.

  size(size_hint(), false);

  // Callbacks.
  
  _p->ok->sig.set(this, ok_callback);
  _p->close->sig.set(this, close_callback);
}

Question::~Question()
{}

void Question::label(const Str & in)
{
  _p->widget->set(in);
  
  size(size_hint(), false);
}

const Str & Question::label() const { return _p->widget->get(); }

void Question::show()
{
  _p->close->take_focus();

  Message_Base::show();
}

void Question::ok_callback(bool)
{
  sig.emit(true);
  
  hide();
}

void Question::close_callback(bool)
{
  hide();
}

}
}
