/*------------------------------------------------------------------------------
 lib/djv_ui/file_browser.h
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef _DJV_UI_FILE_BROWSER_H
#define _DJV_UI_FILE_BROWSER_H

#include <djv_ui/button.h>
#include <djv_ui/columns.h>
#include <djv_ui/file_browser.h>
#include <djv_ui/menu.h>
#include <djv_ui/style.h>
#include <djv_ui/text.h>
#include <djv_image/io.h>
#include <djv_base/file.h>
#include <FL/Fl_Browser_.H>
#include <list>

namespace djv_ui {
namespace file_browser {

class Item
{
public:

  Item();
  ~Item();
  
  void set(const File &, Item * prev, Item * next, bool odd);

  const File & get() const { return _value; }
  Item * prev() const { return _prev; }
  Item * next() const { return _next; }
  bool odd() const { return _odd; }

  Fl_Image * image(int size, IMAGE) const;
  const djv_image::Info & image_info() const { return _image_info; }
  const djv_image::Time & image_time() const { return _image_time; }
  bool image_valid() const { return _image_valid; }
  bool image_init() const { return _image_init; }

private:
  
  File _value;
  Item * _prev, * _next;
  bool _odd;
  Fl_Image * _image;
  djv_image::Info _image_info;
  djv_image::Time _image_time;
  bool _image_valid, _image_init;
};

class Browser : public Fl_Browser_, public widget::Base
{
public:

  Browser();
  ~Browser();
  
  // Value.
  
  void set(const List<File> &, IMAGE image);
  
  File get() const;
  
  Signal<const File &> sig;
  Signal<const File &> action_sig;
  
  // Columns.

  void column_list(const List<int> &);
  
  // Widget.
  
  int handle(int);
  void show();
  void draw();

  void * item_first() const
    { return _item_list.size() ? (void *)&_item_list.front() : 0; }
  void * item_next(void * in) const
    { return in ? static_cast<file_browser::Item *>(in)->next() : 0; }
  void * item_prev(void * in) const
    { return in ? static_cast<file_browser::Item *>(in)->prev() : 0; }
  int item_height(void *) const;
  int item_width(void *) const { return 0; }
  void item_draw(void *, int, int, int, int) const;

  // Layout.
  
  void dirty();
  void geom(const Box2i &);
  
  const Box2i & geom() const { return widget::Base::geom(); }
  void resize(int x, int y, int w, int h) { geom(Box2i(x, y, w, h)); }
  int visible() const { return Fl_Browser_::visible(); }
  
  // Debugging.

  Str dbg_name() const { return "djv::ui::file_browser::Browser"; }

private:
  
  void item_del();
  
  DJV_FL_WIDGET_CALLBACK(Browser, widget_callback);
  DJV_FL_CALLBACK(Browser, image_callback);

  List<file_browser::Item> _item_list;
  int _column_list [_SORT_SIZE];
  int _size_text, _size_icon, _size_thumbnail, _margin;
  IMAGE _image;
  std::list<file_browser::Item *> _image_work_list;
};

struct Dialog::_Dialog
{
  menu::Widget * menu;
  List<int> menu_recent, menu_image, menu_seq, menu_sort, menu_bookmark;
#if defined(WIN32)
  List<int> menu_drive;
#endif
  text::Edit * widget, * dir;
  button::Tool * up, * prev, * reload;
  Browser * browser;
  columns::Widget * columns;
  button::Check * pin;
};

List<int> columns_size_list();

}
}

#endif

