/*------------------------------------------------------------------------------
 lib/djv_ui/image_io.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG
//#define _X_PLUGIN

#include <djv_ui/image_io.h>
#include <djv_ui/image_io_cineon.h>
#include <djv_ui/image_io_dpx.h>
#include <djv_ui/image_io_lut.h>
#include <djv_ui/image_io_ppm.h>
#include <djv_ui/image_io_sgi.h>
#include <djv_ui/image_io_tga.h>
#include <djv_base/sys.h>

namespace djv_ui {
namespace image_io {

Base::Base() :
  sig(this)
{}

Base::~Base()
{}

Load_Factory::Load_Factory()
{
  DJV_DBG_OBJ("Load_Factory::Load_Factory");
  
  add(new image_io_cineon::Load);
  add(new image_io_dpx::Load);
  add(new image_io_lut::Load);

#if ! defined(_X_PLUGIN)
  set(sys::search_path());
#endif
}

Load_Factory::~Load_Factory()
{
  DJV_DBG_OBJ("Load_Factory::~Load_Factory");
  
  const List<Load *> & list = this->list();
  for (uint i = 0; i < list.size(); ++i)
    list[i]->widget::Widget::del();
}

Load * Load_Factory::get(djv_image::io::Load * in) const
{
  const List<plugin::Plugin *> & list = Factory::list();
  
  const Str name = in->name();
  for (uint i = 0; i < list.size(); ++i)
    if (list[i]->name() == name)
      return static_cast<Load *>(list[i]->copy());
  return 0;
}

List<Load *> Load_Factory::list() const
{
  const List<plugin::Plugin *> & list = Factory::list();
  
  List<Load *> out(list.size());
  for (uint i = 0; i < list.size(); ++i)
    out[i] = static_cast<Load *>(list[i]);
  return out;
}

Save_Factory::Save_Factory()
{
  DJV_DBG_OBJ("Save_Factory::Save_Factory");
  
  add(new image_io_cineon::Save);
  add(new image_io_dpx::Save);
  add(new image_io_lut::Save);
  add(new image_io_ppm::Save);
  add(new image_io_sgi::Save);
  add(new image_io_tga::Save);
  
#if ! defined(_X_PLUGIN)
  set(sys::search_path());
#endif
}

Save_Factory::~Save_Factory()
{
  DJV_DBG_OBJ("Save_Factory::~Save_Factory");
  
  const List<Save *> & list = this->list();
  for (uint i = 0; i < list.size(); ++i)
    list[i]->widget::Widget::del();
}

Save * Save_Factory::get(djv_image::io::Save * in) const
{
  const List<plugin::Plugin *> & list = Factory::list();
  
  const Str name = in->name();
  for (uint i = 0; i < list.size(); ++i)
    if (list[i]->name() == name)
      return static_cast<Save *>(list[i]->copy());
  return 0;
}

List<Save *> Save_Factory::list() const
{
  const List<plugin::Plugin *> & list = Factory::list();
  
  List<Save *> out(list.size());
  for (uint i = 0; i < list.size(); ++i)
    out[i] = static_cast<Save *>(list[i]);
  return out;
}

}
}
