/*------------------------------------------------------------------------------
 lib/djv_ui/image_io_lut.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/choice.h>
#include <djv_ui/group.h>
#include <djv_ui/image_io_lut.h>
#include <djv_ui/label.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/style.h>

namespace djv_ui {
namespace image_io_lut {
using namespace djv_image::io_lut;

struct Load::_Load
{
  choice::Widget * spec;
  choice::Widget * type;
};
  
Load::Load() :
  _plugin(0),
  _p(new _Load)
{
  // Widgets.
  
  group::Box * spec_group =
    new group::Box(label_spec_group);
  _p->spec =
    new choice::Widget(djv_image::io_lut::label_spec(), label_spec);

  group::Box * type_group =
    new group::Box(label_type_group);
  _p->type =
    new choice::Widget(djv_image::io_lut::label_type(), label_type);
  
  // Layout.
  
  Layout_V * layout, * layout_group;
  
  layout = new Layout_V(this);

  layout->add(spec_group);
  layout_group = new Layout_V(spec_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(_p->spec);

  layout->add(type_group);
  layout_group = new Layout_V(type_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(_p->type);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _p->spec->sig.set(this, spec_callback);
  _p->type->sig.set(this, type_callback);
}

Load::~Load()
{}

plugin::Plugin * Load::copy() const { return new Load; }

const Str & Load::name() const { return djv_image::io_lut::name; }

void Load::set(djv_image::io::Plugin * in)
{
  _plugin = in;
  
  widget_update();
}

void Load::spec_callback(int in)
{
  if (! _plugin) return;
  
  _option.spec = SPEC(in);
  
  plugin_update();
  sig.emit(_plugin);
}

void Load::type_callback(int in)
{
  if (! _plugin) return;
  
  _option.type = TYPE(in);
  
  plugin_update();
  sig.emit(_plugin);
}

void Load::plugin_update()
{
  callbacks(false);
  
  if (_plugin)
  {
    Str tmp;
    tmp << _option.spec;
    _plugin->option(_plugin->option_list()[LOAD_SPEC], &tmp);
    tmp << _option.type;
    _plugin->option(_plugin->option_list()[LOAD_TYPE], &tmp);
  }
  
  callbacks(true);
}

void Load::widget_update()
{
  callbacks(false);

try
{  
  if (_plugin)
  {
    Str tmp;
    tmp = _plugin->option(_plugin->option_list()[LOAD_SPEC]);
    tmp >> _option.spec;
    tmp = _plugin->option(_plugin->option_list()[LOAD_TYPE]);
    tmp >> _option.type;
  }
}
catch (Str) {}

  _p->spec->set(_option.spec);
  _p->type->set(_option.type);
  
  callbacks(true);
}

struct Save::_Save
{
  choice::Widget * spec;
};

Save::Save() :
  _plugin(0),
  _p(new _Save)
{
  // Widgets.
  
  group::Box * spec_group =
    new group::Box(label_spec_group);
  _p->spec =
    new choice::Widget(djv_image::io_lut::label_spec(), label_spec);
  
  // Layout.

  Layout_V * layout, * layout_group;
  
  layout = new Layout_V(this);

  layout->add(spec_group);
  layout_group = new Layout_V(spec_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(_p->spec);

  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _p->spec->sig.set(this, spec_callback);
}

Save::~Save()
{}

plugin::Plugin * Save::copy() const { return new Save; }

const Str & Save::name() const { return djv_image::io_lut::name; }

void Save::set(djv_image::io::Plugin * in)
{
  _plugin = in;
  
  widget_update();
}

void Save::spec_callback(int in)
{
  if (! _plugin) return;
  
  _option.spec = SPEC(in);
  
  plugin_update();
  sig.emit(_plugin);
}

void Save::plugin_update()
{
  callbacks(false);
  
  if (_plugin)
  {
    Str tmp;
    tmp << _option.spec;
    _plugin->option(_plugin->option_list()[SAVE_SPEC], &tmp);
  }
  
  callbacks(true);
}

void Save::widget_update()
{
  callbacks(false);

try
{
  if (_plugin)
  {
    Str tmp;
    tmp = _plugin->option(_plugin->option_list()[SAVE_SPEC]);
    tmp >> _option.spec;
  }
}
catch (Str) {}

  _p->spec->set(_option.spec);
  
  callbacks(true);
}

const Str
  label_spec_group = "Spec",
  label_spec = "Value:",
  label_type_group = "Type",
  label_type = "Value:";

}
}

