/*------------------------------------------------------------------------------
 lib/djv_ui/layout.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/layout.h>
#include <djv_ui/style.h>
#include <djv_ui/widget.h>
#include <FL/Fl.H>
#include <FL/Fl_Group.H>
//#include <algorithm>
#include <stdio.h>

namespace djv_ui {
namespace layout {

Spacer::Spacer() :
  Item(SPACER)
{}

Layout::Layout(Item * parent) :
  Item(LAYOUT),
  _spacing(-1),
  _margin(-1)
{
  DJV_DBG_OBJ("Layout::Layout");
  
  align(CENTER, CENTER);
  
  // XXX Parent.

  if (parent)
  {
    if (WIDGET == parent->item())
      static_cast<widget::Base *>(parent)->layout(this);
    else if (LAYOUT == parent->item())
      static_cast<Layout *>(parent)->add(this);
    this->parent(parent);
  }
}

Layout::~Layout()
{
  DJV_DBG_OBJ("Layout::~Layout");
  DJV_DBG_PRN("item = " << item());
  
  del();
}

void Layout::add(Item * in)
{
  if (! in) return;
  
  insert(in, int(_list.size()));
}

void Layout::insert(Item * in, int index)
{
  if (! in) return;
  
  DJV_DBG_OBJ("Layout::insert");
  DJV_DBG_PRN("in = " << in->dbg_name());
  DJV_DBG_PRN("parent = " << (parent() ? parent()->dbg_name() : ""));
  DJV_DBG_PRN("index = " << index);
  DJV_DBG_PRN("item = " << in->item());
  
  List<Item *>::iterator i = _list.begin();
  int j = 0;
  for (; j < index; ++j, ++i) ;
  _list.insert(i, in);
  
  // XXX Reparent item.

  if (in->parent())
  {
    if (WIDGET == in->parent()->item())
    {
      if (LAYOUT == in->item())
        static_cast<widget::Base *>(in->parent())->layout(0);
    }
    else if (LAYOUT == in->parent()->item())
    {
      static_cast<Layout *>(in->parent())->remove(in);
    }
  }
  
  if (Fl_Widget * widget = dynamic_cast<Fl_Widget *>(in))
  {
    if (widget->parent())
    {
      DJV_DBG_PRN("fltk remove");
      widget->parent()->remove(widget);
      widget->parent(0);
    }
    Item * parent = this;
    while (WIDGET != parent->item())
    {
      parent = parent->parent();
      if (! parent) break;
    }
    if (Fl_Group * group = dynamic_cast<Fl_Group *>(parent))
    {
      DJV_DBG_PRN("fltk add");
      group->insert(*widget, j);
    }
  }
  
  in->parent(this);

#if defined(DJV_DBG)
  //dbg();
#endif
}

Item * Layout::remove(int in)
{
  DJV_DBG_OBJ("Layout::remove");
  DJV_DBG_PRN("in = " << in);
  
  List<Item *>::iterator i = _list.begin();
  for (int j = 0; j < in; ++j, ++i) ;
  Item * item = *i;
  _list.erase(i);
  DJV_DBG_PRN("item = " << item->dbg_name());
  
  // XXX

  if (Fl_Widget * widget = dynamic_cast<Fl_Widget *>(item))
    if (widget->parent())
    {
      DJV_DBG_PRN("fltk remove");
      widget->parent()->remove(widget);
      widget->parent(0);
    }
  
  item->parent(0);
  
  return item;
}

Item * Layout::remove(Item * in)
{
  Item * out = 0;
  List<Item *>::iterator i = _list.begin();
  for (List<Item *>::const_iterator end = _list.end(); i != end; ++i)
    if (in == *i)
    {
      out = *i;
      _list.erase(i);
      break;
    }
  return out;
}

void Layout::del(int in)
{
  DJV_DBG_OBJ("Layout::del");
  DJV_DBG_PRN("in = " << in);
  
  Item * item = remove(in);
  
  // XXX

  if (WIDGET == item->item())
    static_cast<widget::Base *>(item)->del();
  else
    delete item;
}

void Layout::del(Item * in)
{
  for (uint i = 0; i < _list.size(); ++i)
    if (in == _list[i])
    {
      del(i);
      break;
    }
}

void Layout::del()
{
  while (_list.size()) del(0);
}

void Layout::spacing(int in)
{
  _spacing = in;
}
  
int Layout::spacing() const
{
  return _spacing != -1 ? _spacing : style::global()->spacing();
}

void Layout::margin(int in)
{
  _margin = in;
}

int Layout::margin() const
{
  return _margin != -1 ? _margin : style::global()->margin();
}

void Layout::dirty()
{
  List<Item *>::iterator i = _list.begin();
  for (List<Item *>::const_iterator end = _list.end(); i != end; ++i)
    (*i)->dirty();
}

int Layout::visible() const
{
  for (
    List<Item *>::const_iterator i = _list.begin(), end = _list.end();
    i != end;
    ++i
  )
    if (SPACER != (*i)->item())
      if ((*i)->visible()) return 1;
  return 0;
}

namespace {

void _dbg(const Item * in, int indent)
{
  const Box2i & geom = in->geom();
  const Str tmp = str::Format("%%%% %%,%%,%%x%%\n").
    arg(Str(indent, ' ')).
    arg(in->dbg_name()).
    arg(geom.x).arg(geom.y).arg(geom.w).arg(geom.h);
  printf(tmp.c_str());
}

}

void Layout::dbg(int indent)
{
//#if defined(DJV_DBG)
  _dbg(this, indent);
  for (uint i = 0; i < _list.size(); ++i)
  {
    if (widget::Base * widget = dynamic_cast<widget::Base *>(_list[i]))
    {
      _dbg(widget, indent + 2);
      if (widget->layout()) widget->layout()->dbg(indent + 2);
    }
    if (Layout * layout = dynamic_cast<Layout *>(_list[i]))
      layout->dbg(indent + 2);
  }
//#endif
}

}
}

