/*------------------------------------------------------------------------------
 lib/djv_ui/int.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/button.h>
#include <djv_ui/label.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/num_int.h>
#include <djv_ui/slider.h>
#include <djv_ui/text.h>
#include <FL/Fl.H>

namespace djv_ui {
namespace num {

struct Int_Edit::_Int_Edit
{
  text::Edit * widget;
};

Int_Edit::Int_Edit() :
  sig(this),
  _p(new _Int_Edit)
{
  // Widgets.
  
  _p->widget = new text::Edit;
  _p->widget->size_str(format_int);
  
  Inc * inc_widget = new Inc;
  
  // Layout.
  
  size(layout::FIXED, layout::FIXED);
  align(layout::LEFT, layout::CENTER);
  
  Layout_H * layout = new Layout_H(this);
  layout->spacing(2);
  layout->margin(0);
  layout->add(_p->widget);
  layout->stretch(_p->widget);
  layout->add(inc_widget);
  
  // Initialize.
  
  value_update();
  
  // Callbacks.
  
  _p->widget->sig.set(this, widget_callback);
  inc_widget->inc_sig.set(this, inc_callback);
  inc_widget->dec_sig.set(this, dec_callback);
}

Int_Edit::~Int_Edit()
{}

int Int_Edit::handle(int in)
{
  if (Int_Base::handle(in)) return 1;
  return widget::Widget::handle(in);
}

layout::Item * Int_Edit::group_align() { return _p->widget->group_align(); }

void Int_Edit::value_callback()
{
  value_update();
  
  sig.emit(get());
}

void Int_Edit::widget_callback(const Str & in)
{
  Int_Base::set(str::str_to_int(in));
  
  value_callback();
}

void Int_Edit::inc_callback(bool) { Int_Base::inc_small_action(); }
void Int_Edit::dec_callback(bool) { Int_Base::dec_small_action(); }

void Int_Edit::value_update()
{
  callbacks(false);
  
  _p->widget->set(str::int_to_str(get()));
  
  callbacks(true);
}

struct Int_Display::_Int_Display
{
  text::Display * widget;
};

Int_Display::Int_Display() :
  _p(new _Int_Display)
{
  // Widgets.
  
  _p->widget = new text::Display;
  _p->widget->size_str(format_int);
  
  // Layout.
  
  size(layout::FIXED, layout::FIXED);
  align(layout::LEFT, layout::CENTER);
  
  Layout_H * layout = new Layout_H(this);
  layout->spacing(0);
  layout->margin(0);
  layout->add(_p->widget);
}

Int_Display::~Int_Display()
{}

void Int_Display::set(int in)
{
  _p->widget->set(str::int_to_str(in));
}

struct Int_Slider::_Int_Slider
{
  _Int_Slider() :
    label(0)
  {}

  text::Edit * widget;
  slider::Int * slider;
  widget::Widget * label;
  button::Tool * _default;
  layout::Horizontal * layout;
};

Int_Slider::Int_Slider(int min, int max, const Str & label) :
  Int_Base(min, max),
  sig(this),
  _p(new _Int_Slider)
{
  // Widgets.
  
  _p->widget = new text::Edit;
  _p->widget->size_str(format_int);
  
  Inc * inc_widget = new Inc;
  
  _p->slider = new slider::Int;
  
  if (! label.empty())
    _p->label = new label::Widget(label);
  
  _p->_default = new button::Tool("tool_x");
  _p->_default->tooltip(base::tooltip_reset);
  _p->_default->deactivate();
  _p->_default->hide();
  
  // Layout.
  
  size(layout::STRETCH, layout::FIXED);
  
  _p->layout = new Layout_H(this);
  _p->layout->spacing(2);
  _p->layout->margin(0);
  if (_p->label)
    _p->layout->add(_p->label);
  _p->layout->add(_p->widget);
  _p->layout->add(inc_widget);
  _p->layout->add(_p->slider);
  _p->layout->stretch(_p->slider);
  _p->layout->add(_p->_default);

  // Initialize.
  
  range_update();
  value_update();

  // Callbacks.
  
  _p->widget->sig.set(this, widget_callback);
  inc_widget->inc_sig.set(this, inc_callback);
  inc_widget->dec_sig.set(this, dec_callback);
  _p->slider->sig.set(this, slider_callback);
  _p->_default->sig.set(this, default_callback);
}

Int_Slider::~Int_Slider()
{}

void Int_Slider::default_valid(bool in)
{
  Int_Base::default_valid(in);
  
  if (in) _p->_default->show();
  else _p->_default->hide();
  update();
}
  
bool Int_Slider::default_valid() const { return Int_Base::default_valid(); }

void Int_Slider::label(widget::Widget * in)
{
  if (_p->label) _p->layout->del(_p->label);
  _p->label = in;
  if (_p->label) _p->layout->insert(_p->label, 0);
  _p->layout->stretch(_p->slider);
  
  update();
}

widget::Widget * Int_Slider::label() const { return _p->label; }
layout::Item * Int_Slider::label_align() { return _p->label; }

void Int_Slider::text_font(const Fl_Font & in)
{
  widget::Widget::text_font(in);
  if (_p->label) _p->label->text_font(in);
}

const Fl_Font & Int_Slider::text_font() const
{
  return widget::Widget::text_font();
}

void Int_Slider::text_color(const Fl_Color & in)
{
  widget::Widget::text_color(in);
  if (_p->label) _p->label->text_color(in);
}

const Fl_Color & Int_Slider::text_color() const
{
  return widget::Widget::text_color();
}

int Int_Slider::handle(int in)
{
  if (Int_Base::handle(in)) return 1;
  return widget::Widget::handle(in);
}

layout::Item * Int_Slider::group_align() { return _p->widget->group_align(); }

void Int_Slider::value_callback()
{
  value_update();
  
  sig.emit(get());
}

void Int_Slider::range_callback()
{
  range_update();
}

void Int_Slider::widget_callback(const Str & in)
{
  Int_Base::set(str::str_to_int(in));
  
  value_callback();
}

void Int_Slider::inc_callback(bool)
{
  inc_small_action();
}

void Int_Slider::dec_callback(bool)
{
  dec_small_action();
}

void Int_Slider::slider_callback(int in)
{
  Int_Base::set(in);
}

void Int_Slider::default_callback(bool)
{
  Int_Base::set(default_value());
}

void Int_Slider::value_update()
{
  callbacks(false);
  
  const int value = get();
  
  _p->widget->set(str::int_to_str(value));
  _p->slider->set(value);
  
  if (default_valid() && value != default_value())
    _p->_default->activate();
  else
    _p->_default->deactivate();

  callbacks(true);
}

void Int_Slider::range_update()
{
  callbacks(false);
  
  _p->slider->range(min(), max());
  _p->slider->inc(inc_small(), inc_large());
  
  callbacks(true);
}

const Str format_int = "00000";

}
}

