/*------------------------------------------------------------------------------
 plugin/djv_exr/ui.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "ui.h"
#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/num_float.h>
#include <djv_ui/style.h>

namespace djv_exr {
namespace ui {

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_ui_image_load() { return new Load; }
DJV_PLUGIN_EXPORT plugin::Plugin * djv_ui_image_save() { return new Save; }
}

struct Load::_Load
{
  choice::Widget * color;
  num::Float_Slider * exposure, * defog, * knee_low, * knee_high, * gamma;
};

Load::Load() :
  _plugin(0),
  _p(new _Load)
{
  DJV_DBG_OBJ("Load::Load");

  // Widgets.
  
  const io::Load_Option default_value;
  
  group::Box * color_group =
    new group::Box(label_color_group);
  _p->color =
    new choice::Widget(djv_exr::label_color(), label_color);
  
  group::Box * exposure_group =
    new group::Box(label_exposure_group);
  _p->exposure =
    new num::Float_Slider(-10.0, 10.0, label_exposure);
  _p->defog =
    new num::Float_Slider(0.0, 0.01, label_exposure_defog);
  _p->knee_low =
    new num::Float_Slider(-3.0, 3.0, label_exposure_knee);
  _p->knee_high =
    new num::Float_Slider(3.5, 7.5);
  _p->knee_high->label(new label::Widget);
  _p->gamma =
    new num::Float_Slider(0.1, 4.0, label_exposure_gamma);

  // Layout.
  
  layout::group(List<layout::Item *>() <<
    _p->exposure <<
    _p->defog <<
    _p->gamma <<
    _p->knee_low <<
    _p->knee_high <<
    _p->gamma);
  
  Layout_V * layout, * layout_group, * layout_v;
  
  layout = new Layout_V(this);

  layout->add(color_group);
  layout_group = new Layout_V(color_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(_p->color);

  layout->add(exposure_group);  
  layout_group = new Layout_V(exposure_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_v = new Layout_V(layout_group);
  layout_v->margin(0);
  layout_v->add(_p->exposure);
  layout_v->add(_p->defog);
  layout_v->add(_p->knee_low);
  layout_v->add(_p->knee_high);
  layout_v->add(_p->gamma);
  
  layout->add_stretch();
  
  // Initialize.

  _p->exposure->inc(0.1, 1.0);
  _p->defog->inc(0.0001, 0.001);
  _p->knee_low->inc(0.1, 1.0);
  _p->knee_high->inc(0.1, 1.0);

  _p->exposure->default_value(default_value.exposure.value);
  _p->defog->default_value(default_value.exposure.defog);
  _p->knee_low->default_value(default_value.exposure.knee_low);
  _p->knee_high->default_value(default_value.exposure.knee_high);
  _p->gamma->default_value(default_value.exposure.gamma);

  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _p->color->sig.set(this, color_callback);
  _p->exposure->sig.set(this, exposure_callback);
  _p->defog->sig.set(this, defog_callback);
  _p->knee_low->sig.set(this, knee_low_callback);
  _p->knee_high->sig.set(this, knee_high_callback);
  _p->gamma->sig.set(this, gamma_callback);
}

plugin::Plugin * Load::copy() const { return new Load; }

const Str & Load::name() const { return djv_exr::name; }

void Load::set(djv_image::io::Plugin * in)
{
  _plugin = in;
  
  widget_update();
}

void Load::color_callback(int in)
  { callback((_option.color = COLOR(in))); }
void Load::exposure_callback(double in)
  { callback((_option.exposure.value = in)); }
void Load::defog_callback(double in)
  { callback((_option.exposure.defog = in)); }
void Load::knee_low_callback(double in)
  { callback((_option.exposure.knee_low = in)); }
void Load::knee_high_callback(double in)
  { callback((_option.exposure.knee_high = in)); }
void Load::gamma_callback(double in)
  { callback((_option.exposure.gamma = in)); }

void Load::callback(bool)
{
  if (! _plugin) return;
  
  plugin_update();
  sig.emit(_plugin);
}

void Load::plugin_update()
{
  callbacks(false);

  if (_plugin)
  {
    Str tmp;
    tmp << _option.color;
    _plugin->option(_plugin->option_list()[io::LOAD_COLOR], &tmp);
    tmp << _option.exposure;
    _plugin->option(_plugin->option_list()[io::LOAD_EXPOSURE], &tmp);
  }

  callbacks(true);
}

void Load::widget_update()
{
  callbacks(false);

try
{  
  if (_plugin)
  {
    Str tmp;
    tmp = _plugin->option(_plugin->option_list()[io::LOAD_COLOR]);
    tmp >> _option.color;
    tmp = _plugin->option(_plugin->option_list()[io::LOAD_EXPOSURE]);
    tmp >> _option.exposure;
  }
}
catch (Str) {}
  
  _p->color->set(_option.color);
  _p->exposure->set(_option.exposure.value);
  _p->defog->set(_option.exposure.defog);
  _p->knee_low->set(_option.exposure.knee_low);
  _p->knee_high->set(_option.exposure.knee_high);
  _p->gamma->set(_option.exposure.gamma);
  
  callbacks(true);
}

struct Save::_Save
{
  choice::Widget * compress;
};

Save::Save() :
  _plugin(0),
  _p(new _Save)
{
  DJV_DBG_OBJ("Save::Save");
  
  // Widgets.
  
  group::Box * compress_group =
    new group::Box(label_compress_group);
  _p->compress =
    new choice::Widget(djv_exr::label_compress(), label_compress);
  
  // Layout.

  Layout_V * layout, * layout_group;
  
  layout = new Layout_V(this);

  layout->add(compress_group);
  layout_group = new Layout_V(compress_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(_p->compress);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _p->compress->sig.set(this, compress_callback);
}

plugin::Plugin * Save::copy() const { return new Save; }

const Str & Save::name() const { return djv_exr::name; }

void Save::set(djv_image::io::Plugin * in)
{
  _plugin = in;
  
  widget_update();
}

void Save::compress_callback(int in)
{
  if (! _plugin) return;
  
  _option.compress = COMPRESS(in);
  
  plugin_update();
  sig.emit(_plugin);
}

void Save::plugin_update()
{
  callbacks(false);

  if (_plugin)
  {
    Str tmp;
    tmp = str::serialize(_option.compress, djv_exr::label_compress());
    _plugin->option(_plugin->option_list()[io::SAVE_COMPRESS], &tmp);
  }

  callbacks(true);
}

void Save::widget_update()
{
  callbacks(false);
  
  if (_plugin)
  {
    Str tmp;
    tmp = _plugin->option(_plugin->option_list()[io::SAVE_COMPRESS]);
    str::serialize(&tmp, &_option.compress, djv_exr::label_compress());
  }
  
  _p->compress->set(_option.compress);
  
  callbacks(true);
}

const Str
  label_color_group = "Color",
  label_color = "Color space:",
  label_exposure_group = "Exposure",
  label_exposure = "Value:",
  label_exposure_defog = "Defog:",
  label_exposure_knee = "Knee:",
  label_exposure_gamma = "Gamma:",
  label_compress_group = "Compress",
  label_compress = "Value:";

}
}

