/*------------------------------------------------------------------------------
 plugin/djv_jpeg/io.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_JPEG_IO_H
#define DJV_JPEG_IO_H

#include "base.h"
#include <djv_image/io.h>

namespace djv_jpeg {
namespace io {

//------------------------------------------------------------------------------
// Options

enum SAVE_OPTION
{
  SAVE_QUALITY
};

struct Save_Option
{
  Save_Option() :
    quality(100)
  {}

  int quality;
};

//------------------------------------------------------------------------------
// Load Plugin

class Load : public djv_image::io::Load
{
public:

  Load();
  ~Load();
  
  plugin::Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  void open(const File &, Info *, Tag *, djv_image::Time *) throw (Error);
  void open(const File &, List<Info> *, Tag *, djv_image::Time *) throw (Error);
  const Image * get(int frame, int layer, data::PROXY) throw (Error);

private:

  void _open(const File &, Info *, Tag *) throw (Error);
  void _close();

  File _file;
  FILE * _f;
  libjpeg::jpeg_decompress_struct _jpeg;
  bool _jpeg_init;
  Jpeg_Error _jpeg_error;
  Image _image;
  Data _tmp;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public djv_image::io::Save
{
public:

  Save();
  ~Save();
  
  plugin::Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  bool option(const Str &, Str *);
  Str option(const Str &) const;
  const List<Str> & option_list() const;

  void open(const File &, const Info &, const Tag &, const djv_image::Time &)
    throw (Error);
  void set(const Image &, int frame) throw (Error);

private:
  
  void _open(const File &, const Tag &) throw (Error);
  void _close();

  Save_Option _option;
  File _file;
  FILE * _f;
  libjpeg::jpeg_compress_struct _jpeg;
  bool _jpeg_init;
  Jpeg_Error _jpeg_error;
  Info _info;
  Image _image;
};

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_save_option();

}
}

#endif

