/*------------------------------------------------------------------------------
 plugin/djv_tiff/base.h

 Image I/O - TIFF
 ----------------
 
 Tagged image file format.

 Requires:
 
 * libtiff - http://www.libtiff.org
 
 Support:
 
 * Image data: L, LA, RGB, RGBA, 8-bit, 16-bit, 32-bit float.
 * Interleaved channels only.
 * File compression.
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_TIFF_BASE_H
#define DJV_TIFF_BASE_H

#include <djv_image/base.h>
#include <tiffio.h>

namespace djv_tiff {
using namespace djv_base;
using namespace djv_image;

//------------------------------------------------------------------------------
// Information

static const Str name = "TIFF";

static const List<Str> extension_list = List<Str>() <<
  ".tiff" <<
  ".tif";

//------------------------------------------------------------------------------
// Options

enum COMPRESS
{
  COMPRESS_NONE,
  COMPRESS_RLE,
  COMPRESS_LZW
};

//------------------------------------------------------------------------------
// Functions

void tiff_init();

void palette_load(
  uint8_t *,
  int size,
  int bytes,
  uint16_t * red,
  uint16_t * green,
  uint16_t * blue
);

//------------------------------------------------------------------------------
// Conversion

Str & operator >> (Str &, COMPRESS &) throw (Str);
Str & operator << (Str &, COMPRESS);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_compress();

}

#endif

