/*------------------------------------------------------------------------------
 plugin/djv_vlut/load.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "io.h"
#include <FL/x.h>

namespace djv_vlut {
namespace io {

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_image_load() { return new Load; }
}

plugin::Plugin * Load::copy() const { return new Load; }

const Str & Load::name() const { return djv_vlut::name; }

const List<Str> & Load::extension_list() const
  { return djv_vlut::extension_list; }

void Load::open(const File & in, Info * info, Tag *, djv_image::Time *)
  throw (Error)
{
  DJV_DBG_OBJ("Load::open");
  
  // Open.

#if defined(WIN32)

  _handle.open();

#endif
  
  // Information.

  int size = 0;

#if defined(WIN32)

  size = 256;

#elif defined(DJV_IRIX)

  int maps = 0, precision = 0, attr = 0;
  if (! XSGIvcQueryScreenGammaMaps(fl_display, fl_screen, &maps))
    throw Error(name(), djv_image::io::error_open);
  if (maps <= 0)
    throw Error(name(), djv_image::io::error_read);

  if (
    ! XSGIvcQueryGammaMap(fl_display, fl_screen, 0, &size, &precision, &attr)
  )
    throw Error(name(), djv_image::io::error_open);

  if (attr & XSGIVC_GMReadLock)
    throw Error(name(), djv_image::io::error_open);

#else

  if (! XF86VidModeGetGammaRampSize(fl_display, fl_screen, &size))
    throw Error(name(), djv_image::io::error_open);
  
#endif

  _info.name = in;
  _info.size = Vec2i(size, 1);
  _info.pixel = pixel::RGB_U16;
  *info = _info;

  _image.set(_info);
  _tmp.set(_info);
}

void Load::open(const File & in, List<Info> * info, Tag * tag,
  djv_image::Time * time) throw (Error)
{
  djv_image::io::Load::open(in, info, tag, time);
}

const Image * Load::get(int, int, data::PROXY) throw (Error)
{
  DJV_DBG_OBJ("Load::get");
  
  // Read.

  const int size = _info.size.x;
  pixel::U16_T * p = reinterpret_cast<pixel::U16_T *>(_tmp.data());

#if defined(WIN32)
  
  ::GetDeviceGammaRamp(_handle(), p);

#elif defined(DJV_IRIX)

  uint16_t * tmp [] = { 0, 0, 0 };
  int _size = 0;
  XSGIvcQueryGammaColors(
    fl_display, fl_screen, 0, XSGIVC_COMPONENT_RED, &_size, tmp);
  XSGIvcQueryGammaColors(
    fl_display, fl_screen, 0, XSGIVC_COMPONENT_GREEN, &_size, tmp + 1);
  XSGIvcQueryGammaColors(
    fl_display, fl_screen, 0, XSGIVC_COMPONENT_BLUE, &_size, tmp + 2);
  mem::copy(tmp[0], p, size * 2);
  mem::copy(tmp[1], p + size, size * 2);
  mem::copy(tmp[2], p + size * 2, size * 2);
  if (tmp[0]) XFree(tmp[0]);
  if (tmp[1]) XFree(tmp[1]);
  if (tmp[2]) XFree(tmp[2]);

#else

  XF86VidModeGetGammaRamp(
    fl_display, fl_screen, size, p, p + size, p + size * 2);
  
#endif
  
#if defined(DJV_DBG)
  //for (int i = 0; i < size; ++i)
  //{
  //  DJV_DBG_PRN(i << " = " << p[i]);
  //}
#endif
  
  data::planar_interleave(_tmp, &_image);
  
  DJV_DBG_PRN("image = " << _image);
  return &_image;
}

}
}

