#define DJV_DBG

#include <djv_base/dbg.h>
#include <djv_base/str.h>

using namespace djv_base;

void format()
{
  DJV_DBG_OBJ("format");
  
  const struct Data { Str a, b; } data [] =
  {
    { "%%%%%%", "abc" },
    { "%%%%", "ab" },
    { "%%%%%%%%", "abc%%" }
  };
  const int data_size = sizeof(data) / sizeof(Data);
  for (int i = 0; i < data_size; ++i)
  {
    const Str tmp = str::Format(data[i].a).arg("a").arg("b").arg("c");
    DJV_DBG_PRN(tmp << " = " << data[i].b);
    DJV_ASSERT(tmp == data[i].b);
  }
}

void format2()
{
  DJV_DBG_OBJ("format2");
  
  const Str tmp =
" a b c\n"
" d e f\n"
"\n"
" g h i";
  //DJV_DBG_PRN(str::split(tmp, '\n', true));
  DJV_DBG_PRN(str::join(str::format(tmp, 80), '|'));
}

void char_case()
{
  DJV_DBG_OBJ("char_case");
  
  DJV_ASSERT("abc" == str::lower("ABC"));
  DJV_ASSERT("ABC" == str::upper("abc"));
}

void join()
{
  DJV_DBG_OBJ("join");
  
  const Str tmp = str::join(List<Str>() << "a" << "b" << "c", " ");
  DJV_DBG_PRN(tmp);
  DJV_ASSERT("a b c" == tmp);
}

void split()
{
  DJV_DBG_OBJ("split");

  const struct Data { Str a; Str split; bool empty; List<Str> b; } data [] =
  {
    { "", "_", false, List<Str>() << "" },
    { "_", "_", false, List<Str>() },
    { "_", "_", true, List<Str>() << "" },
    { "__", "_", false, List<Str>() },
    { "__", "_", true, List<Str>() << "" << "" },
    { "a", "_", false, List<Str>() << "a" },
    { "abc", "_", false, List<Str>() << "abc" },
    { "a_b", "_", false, List<Str>() << "a" << "b" },
    { "a_b_c", "_", false, List<Str>() << "a" << "b" << "c" },
    { "_a_b_c", "_", false, List<Str>() << "a" << "b" << "c" },
    { "a_b_c_", "_", false, List<Str>() << "a" << "b" << "c" },
    { "__a__b__c", "__", false, List<Str>() << "a" << "b" << "c" },
    { "a__b__c__", "__", false, List<Str>() << "a" << "b" << "c" },
    { "a_bc_def", "_", false, List<Str>() << "a" << "bc" << "def" },
    { "a__b__c", "_", true, List<Str>() << "a" << "" << "b" << "" << "c" }
  };
  const int data_size = sizeof(data) / sizeof(Data);
  for (int i = 0; i < data_size; ++i)
  {
    const List<Str> tmp = str::split(data[i].a, data[i].split, data[i].empty);
    DJV_DBG_PRN(data[i].a << " = " << tmp);
    DJV_ASSERT(data[i].b == tmp);
  }
}

void replace()
{
  DJV_DBG_OBJ("replace");
  
  const Str data = "a b c";
  const Str tmp = str::replace(data, ' ', '_');
  DJV_DBG_PRN(data << " = " << tmp);
  DJV_ASSERT("a_b_c" == tmp);
}

void convert()
{
  DJV_DBG_OBJ("convert");

  const struct Int_Data { Str a; int b; Str c; } int_data [] =
  {
    { "-1", -1, "-1" },
    { "0", 0, "0" },
    { ".1", 0, "0" },
    { "1", 1, "1" },
    { "1.1", 1, "1" },
    { "10", 10, "10" },
    { "100", 100, "100" },
    { "1000", 1000, "1000" }
  };
  const int int_size = sizeof(int_data) / sizeof(Int_Data);
  for (int i = 0; i < int_size; ++i)
  {
    const int tmp = str::str_to_int(int_data[i].a);
    DJV_DBG_PRN("int: " << int_data[i].a << " = " << tmp);
    DJV_ASSERT(int_data[i].b == tmp);
    DJV_ASSERT(int_data[i].c == str::int_to_str(tmp));
  }

  const double float_data [] = { -1.0, 0.0, 1.0, 10.0, 100.0, 1000.0 };
  const int float_size = sizeof(float_data) / sizeof(double);
  for (int i = 0; i < float_size; ++i)
  {
    const double tmp = str::str_to_float(str::float_to_str(float_data[i]));
    DJV_DBG_PRN("float: " << float_data[i] << " = " << tmp);
    DJV_ASSERT(float_data[i] == tmp);
  }

  const Str str_data [] = { "abc", "a b c" };
  const int str_size = sizeof(str_data) / sizeof(Str);
  for (int i = 0; i < str_size; ++i)
  {
    Str tmp = str::serialize(str_data[i]);
    Str tmp2;
    tmp >> tmp2;
    DJV_DBG_PRN("str: " << str_data[i] << " = " << tmp2);
    DJV_ASSERT(str_data[i] == tmp2);
  }
}

int main(int argc, char ** argv)
{
  format();
  format2();
  char_case();
  join();
  split();
  replace();
  convert();

  return 0;
}

