#define DJV_DBG

#include <djv_base/vec.h>

using namespace djv_base;

void components()
{
  DJV_DBG_OBJ("components");

  Vec2i vec2i(1, 2);
  Vec2f vec2f(1.0, 2.0);
  DJV_DBG_PRN("vec2i = " << vec2i);
  DJV_DBG_PRN("vec2f = " << vec2f);  
  DJV_ASSERT(1 == vec2i.x);
  DJV_ASSERT(2 == vec2i.y);
  DJV_ASSERT(1.0 == vec2f.x);
  DJV_ASSERT(2.0 == vec2f.y);

  DJV_ASSERT(Vec2i(Vec2f(1.0, 2.0)) == Vec2i(1, 2));
  DJV_ASSERT(Vec2f(Vec2i(1, 2)) == Vec2f(1.0, 2.0));
}

void abs()
{  
  DJV_DBG_OBJ("abs");

  DJV_ASSERT(vec::abs(Vec2i(-1, -2)) == Vec2i(1, 2));
}

void swap()
{  
  DJV_DBG_OBJ("swap");

  DJV_ASSERT(vec::swap(Vec2i(1, 2)) == Vec2i(2, 1));
}

void range()
{  
  DJV_DBG_OBJ("min, max, clamp");

  DJV_ASSERT(vec::min(Vec2i(1, 2), Vec2i(3, 4)) == Vec2i(1, 2));
  DJV_ASSERT(vec::max(Vec2i(1, 2), Vec2i(3, 4)) == Vec2i(3, 4));
  DJV_ASSERT(
    vec::clamp(Vec2i(-10, 10), Vec2i(1, 2), Vec2i(3, 4)) == Vec2i(1, 4));
}

void size_valid()
{    
  DJV_DBG_OBJ("size_valid");
  
  DJV_ASSERT(vec::size_valid(Vec2i(1, 2)) && ! vec::size_valid(Vec2i(-1, -2)));
}

void length()
{
  DJV_DBG_OBJ("length");
  
  DJV_ASSERT(vec::length(Vec2i(0, 2)) == 2);
}

void normalize()
{
  DJV_DBG_OBJ("normalize");
  
  DJV_ASSERT(vec::normalize(Vec2f(0.0, 2.0)) == Vec2f(0.0, 1.0));
}

void dot()
{
  DJV_DBG_OBJ("dot");
  
  DJV_ASSERT(vec::dot(Vec2f(1.0, 0.0), Vec2f(0.0, 1.0)) == 0.0);
}

void aspect()
{
  DJV_DBG_OBJ("aspect");
  
  DJV_ASSERT(vec::aspect(Vec2i(2, 1)) == 2.0);
}

void arithmetic()
{
  DJV_DBG_OBJ("arithemtic");
  
  Vec2i vec2i = Vec2i(1, 2);
  vec2i += Vec2i(1, 2);
  DJV_ASSERT(Vec2i(2, 4) == vec2i);
  vec2i -= Vec2i(1, 2);
  DJV_ASSERT(Vec2i(1, 2) == vec2i);
  vec2i *= Vec2i(1, 2);
  DJV_ASSERT(Vec2i(1, 4) == vec2i);
  vec2i /= Vec2i(1, 2);
  DJV_ASSERT(Vec2i(1, 2) == vec2i);

  DJV_ASSERT(Vec2i(1, 2) + Vec2i(1, 2) == Vec2i(2, 4));
  DJV_ASSERT(Vec2i(1, 2) - Vec2i(1, 2) == Vec2i(0, 0));
  DJV_ASSERT(Vec2i(1, 2) * Vec2i(1, 2) == Vec2i(1, 4));
  DJV_ASSERT(Vec2i(1, 2) / Vec2i(1, 2) == Vec2i(1, 1));
  
  DJV_ASSERT(-Vec2i(1, 2) == Vec2i(-1, -2));
}

void comparison()
{
  DJV_DBG_OBJ("comparison");
  
  DJV_ASSERT(Vec2i(1, 2) == Vec2i(1, 2));
  DJV_ASSERT(Vec2i(1, 2) != Vec2i(0, 0));
  DJV_ASSERT(Vec2i(1, 2) <  Vec2i(2, 3));
  DJV_ASSERT(Vec2i(1, 2) <= Vec2i(1, 2));
  DJV_ASSERT(Vec2i(2, 3) >  Vec2i(1, 2));
  DJV_ASSERT(Vec2i(1, 2) >= Vec2i(1, 2));
}

void conversion()
{
  DJV_DBG_OBJ("conversion");
  
  Vec2i vec2i = vec::convert<double, int, 2>(Vec2f(1.0, 2.0));
  DJV_ASSERT(vec2i == Vec2i(1, 2));
  vec2i = vec::ceil<double, int, 2>(Vec2f(0.5, 0.5));
  DJV_ASSERT(vec2i == Vec2i(1, 1));
  vec2i = vec::floor<double, int, 2>(Vec2f(0.5, 0.5));
  DJV_ASSERT(vec2i == Vec2i(0, 0));

  DJV_ASSERT("1 2" == str::label(Vec2i(1, 2)));
}

int main(int argc, char ** argv)
{
  components();
  abs();
  swap();
  range();
  size_valid();
  length();
  normalize();
  dot();
  aspect();
  arithmetic();
  comparison();
  conversion();
  
  return 0;
}

