#include <djv_ui/layout_row.h>
#include <djv_ui/window.h>
#include <djv_ui/button.h>
#include <djv_ui/tab.h>
#include <djv_ui/color.h>
#include <djv_ui/app.h>

class App : public djv_ui::app::App
{
public:

  App(int & argc, char ** argv);
  ~App();

private:

  void add();
  void del();
  
  DJV_CALLBACK(App, widget_callback, int)
  {
    print(djv_base::str::Format("tab = %%").arg(in));
  }
  DJV_CALLBACK(App, add_callback, bool) { add(); }
  DJV_CALLBACK(App, del_callback, bool) { del(); }
  DJV_CALLBACK(App, exit_callback, bool) { exit(); }

  djv_ui::tab::Widget * _widget;
  djv_ui::window::Widget * _window;
};

App::App(int & argc, char ** argv) :
  djv_ui::app::App("tab", argc, argv),
  _window(0)
{
  using namespace djv_base;
  using namespace djv_ui;

  valid(true);
  
  // Widgets.
   
  _widget = new tab::Widget;

  button::Tool * add_button = new button::Tool("tool_add");
  add_button->type(button::Tool::REPEAT);
  button::Tool * del_button = new button::Tool("tool_x");
  del_button->type(button::Tool::REPEAT);

  group::Frame * frame = new group::Frame;
  frame->edges(false, false, true, false);

  _window = new window::Widget(name());
  
  // Layout.
  
  Layout_V * layout = new Layout_V(_window);
  layout->spacing(0);
  layout->margin(0);
  layout->add(_widget);
  layout->stretch(_widget);
  
  layout->add(frame);
  Layout_H * layout_h = new Layout_H(frame);
  layout_h->add_stretch();
  layout_h->add(add_button);
  layout_h->add(del_button);
  
  // Initialize.
  
  add();
  
  _window->size(Vec2i(300, 300));
  _window->show();
  
  // Callbacks.
  
  _widget->sig.set(this, widget_callback);
  add_button->sig.set(this, add_callback);
  del_button->sig.set(this, del_callback);
  _window->close_sig.set(this, exit_callback);
}

App::~App()
{
  if (_window) _window->del();
}

void App::add()
{
  _widget->tab_add(
    djv_base::str::Format("tab %%").arg(_widget->tab_size()),
    new djv_ui::color::Widget
  );
}

void App::del()
{
  if (! _widget->tab_size()) return;
  
  _widget->tab_del(_widget->get());
}

int main(int argc, char ** argv) { return App(argc, argv).exec(); }

